@@ -3,15 +3,51 @@
 #Script to bootstrap 3Depict cross-compilation under debian
 # and debian-like systems	
 # Note that building wx requires ~8GB of ram (or swap)
-BASE=`pwd`
-#HOST_VAL=x86_64-w64-mingw32 #For mingw64
-#HOST_VAL=i686-w64-mingw32
 
-if [ x$HOST_VAL = x"" ] ; then
-	echo "Please uncomment the right HOST_VAL for your system"
+#--- Determine which system we wish to build for
+#HOST_VAL=x86_64-w64-mingw32 #For mingw64 (windows 64 bit)
+#HOST_VAL=i686-w64-mingw32 #For mingw32 (Windows 32 bit)
+
+if [ ! -f host_val ] ; then
+	echo "Please select 32 or 64 bit by typing \"32\" or \"64\" (32/64)"
+	read HOST_VAL
+	
+	case $HOST_VAL in
+		32)
+			HOST_VAL="i686-w64-mingw32"
+			;;
+		64)
+			HOST_VAL="x86_64-w64-mingw32"
+			;;
+		
+		*) 
+			echo "Didn't understand HOST_VAL. You can override this by editing the script"
+			exit 1
+		;;
+	esac
+
+	#Save for next run
+	echo $HOST_VAL > host_val
+else
+	HOST_VAL=`cat host_val`
+fi
+
+
+if [ $HOST_VAL != "x86_64-w64-mingw32" ] && [ $HOST_VAL != i686-w64-mingw32 ] ; then
+	echo "Unknown HOST_VAL"
 	exit 1
 fi
 
+#----
+
+if [ ! -d code/3Depict ] || [ ! -f code/3Depict/src/3Depict.cpp ] ; then
+	echo "3Depict code dir, \"code/3Depict\", appears to be missing. Please place 3Depict source code in this location"
+	echo "Aborting"
+	exit 1
+fi
+
+
+BASE=`pwd`
 PREFIX=/
 NUM_PROCS=4
 
@@ -24,21 +60,25 @@ if [ `id -u` -eq 0 ]; then
 fi
 #1) Filezilla wxwidgets patch for 64 bit support under mingw
 #2) own patch for fixing wx-config's lack of sysroot support
-PATCHES_WXWIDGETS_PRE="wxWidgets-2.8.12-mingw64-1.patch"
+PATCHES_WXWIDGETS_PRE="wxWidgets-2.8.12-mingw64-1.patch configure-wxbool-patch"
 PATCHES_WXWIDGETS_POST="wx-config-sysroot.patch"
 #1) Zlib no longer needs to explicitly link libc, and will fail if it tries
 PATCHES_ZLIB="zlib-no-lc.patch"
 #1) Override some configure patches to bypass false positive failures
-#2) mingw32 specific patch - mingw32 won't link with overridden prototypes
-PATCHES_FTGL="ftgl-override-configure ftgl-mingw32-prototype"
-
+PATCHES_FTGL="ftgl-disable-doc"
+PATCHES_FTGL_POSTCONF="ftgl-override-configure-2"
 #1) gettext-tools fails in various places, but we don't actually need it, so turn it off
 #2) gettext fails to correctly determine windows function call prefix.
 #   should be fixed for gettext > 0.18.1.1 ?
 #   https://lists.gnu.org/archive/html/bug-gettext/2012-12/msg00071.html
-PATCHES_GETTEXT="gettext-disable-tools gettext-win32-prefix"
+PATCHES_GETTEXT="gettext-disable-tools"    #gettext-win32-prefix
 
-PATCH_LIST="$PATCHES_WXWIDGETS_PRE $PATCHES_WXWIDGETS_POST $PATCHES_GSL $PATCHES_ZLIB $PATCHES_LIBPNG $PATCHES_GETTEXT $PATCHES_FTGL"
+PATCHES_GLEW="glew-makefile.base"
+
+#Disable broken build for "widgets" directory, which we don't need
+PATCHES_MATHGL="mathgl-disable-widgets"
+
+PATCH_LIST="$PATCHES_WXWIDGETS_PRE $PATCHES_WXWIDGETS_POST $PATCHES_GSL $PATCHES_ZLIB $PATCHES_LIBPNG $PATCHES_GETTEXT $PATCHES_FTGL $PATCHES_GLEW $PATCHES_MATHGL $PATCHES_FTGL_POSTCONF"
 
 BUILD_STATUS_FILE="$BASE/build-status"
 PATCH_STATUS_FILE="$BASE/patch-status"
@@ -57,7 +97,7 @@ function applyPatches()
 {
 	for i in $APPLY_PATCH_ARG
 	do
-		if [ x`cat $PATCH_STATUS_FILE | grep "$i"` != x"" ] ; then
+		if [ x"`cat $PATCH_STATUS_FILE | grep -x "$i"`" != x"" ] ; then
 			echo "Patch already applied :" $i
 			continue
 		fi
@@ -76,42 +116,133 @@ function applyPatches()
 
 function install_mingw()
 {
-	echo "Requesting install of mingw :" $MINGW_PACKAGES
 
+	echo "Checking mingw install"
 	#install mingw and libtool (we will need it...)
-	sudo apt-get install $MINGW_PACKAGES libtool
 
-	if [ $? -ne 0] ; then
-		echo "Mingw installation failed".
-		exit 1
+	GET_PACKAGES="";
+	for i in $MINGW_PACKAGES
+	do
+		if [ x`apt-cache pkgnames --installed $i` != x"$i" ] ; then
+			GET_PACKAGES="$GET_PACKAGES $i";
+		fi
+	done
+
+	if [ x"$GET_PACKAGES" != x"" ] ; then
+		echo "Requesting install of mingw :" $GET_PACKAGES
+		sudo apt-get install $GET_PACKAGES libtool || { echo "Mingw install failed";  exit 1 ; }
 	fi
+
 }
 
 function grabDeps()
 {
 	pushd deps 2>/dev/null
-	DEB_PACKAGES="expat freetype ftgl gettext gsl libjpeg8 libpng libxml2 mathgl qhull tiff3 wxwidgets2.8 zlib nsis"
-	apt-get source $DEB_PACKAGES
 
+	DEB_PACKAGES="expat freetype ftgl gettext gsl libpng libxml2 mathgl qhull tiff wxwidgets2.8 zlib glew"
+	if [ x$DIST_NAME == x"Ubuntu" ] || [ x$DIST_NAME == x"LinuxMint" ] ; then 
+		LIBJPEGNAME="libjpeg6b"
+	else
+		LIBJPEGNAME="libjpeg8"
+	
+	fi
+	DEB_PACKAGES="$DEB_PACKAGES $LIBJPEGNAME"
+
+	GET_PACKAGES=""
+	for i in $DEB_PACKAGES
+	do
+		FNAME=`ls packages/${i}_*.orig.* 2> /dev/null`
+		#If filename is empty, we will need to retreive it from
+		# interwebs
+		if [ x"$FNAME" == x"" ] ; then
+			GET_PACKAGES="${GET_PACKAGES} $i"
+		fi
+	done
+
+	#grab packages if they are not already on-disk
+	if [ x"$GET_PACKAGES" != x"" ] ; then
+		apt-get source $GET_PACKAGES
+
+		for i in $GET_PACKAGES
+		do
+			grep -v $i ../build-status > tmp
+			mv tmp ../build-status
+			
+			grep -v $i ../patch-status > tmp
+			mv tmp ../patch-status
+
+
+		done
+	fi
+	
 	if [ $? -ne 0 ] ; then
 		echo "apt-get source failed... Maybe check internet connection, then try updating package database, then re-run?"
 		exit 1
 	fi
 
-	mv *.orig.* *.debian.* *.dsc *.diff.* packages 
+	#Move debian stuff into packages folder
+	if [ x"$GET_PACKAGES" != x"" ] ; then
+		mv *.orig.* *.debian.* *.dsc *.diff.* packages 
+	fi
 
-	if [ ! -f libiconv-1.14.tar.gz ] ; then
-		wget "ftp://ftp.gnu.org/gnu/libiconv/libiconv-1.14.tar.gz" || { echo "Libiconv download failed "; exit 1; }
-	else
-		if [ x`sha1sum libiconv-1.14.tar.gz | awk '{print $1}'` != x"be7d67e50d72ff067b2c0291311bc283add36965" ] ; then
-			echo "SHA1 sum mismatch for libiconv"
-			exit 1
+	#Check that we have untarred all the required packages
+	# (eg we downloaded one, and wiped it at some stage)
+	# if not, pull the package out, and re-build it
+	GET_PACKAGES=""
+	for i in $DEB_PACKAGES
+	do
+		#if we have a package file (dsc), and no folder, add it to the list of packages
+		FNAME=`ls packages/$i*dsc 2> /dev/null` 
+		DNAME=`ls -ld $i* | grep ^d | awk '{print $NF}'`
+		if [ x"$FNAME" != x"" ] && [ x"$DNAME" == x""  ] ; then
+			GET_PACKAGES="${GET_PACKAGES} $i"
 		fi
+	done
+	
+	#Unpack pre-existing package
+	for i in $GET_PACKAGES
+	do
+		mv packages/$i*.* . || { echo "existing package extraction failed "; exit 1; } 
+		dpkg-source -x $i*dsc
+		#move package back
+		mv ${i}_*.* packages/
+		
+		#wipe record of any patches for this package
+		grep -v $i ../patch-status  > tmp
+		mv tmp ../patch-status
+		
+		#wipe record of build
+		grep -v $i ../build-status  > tmp
+		mv tmp ../build-status
+	done
+
+	#extract libiconv if needed
+	#--
+	LIBICONV=libiconv-1.14
+	if [ ! -f packages/${LIBICONV}.tar.gz ] ; then
+		wget "http://ftp.gnu.org/gnu/libiconv/libiconv-1.14.tar.gz" || { echo "Libiconv download failed "; exit 1; }
+		mv ${LIBICONV}.tar.gz packages/
+	fi
+
+	#Check for SHA1 goodness
+	if [ x`sha1sum packages/${LIBICONV}.tar.gz | awk '{print $1}'` != x"be7d67e50d72ff067b2c0291311bc283add36965" ] ; then
+		echo "SHA1 sum mismatch for libiconv"
+		exit 1
 	fi
 	
-	tar -zxf libiconv-1.14.tar.gz || { echo "failed extracting iconv "; exit 1; }
+	#Extract
+	if [ ! -d ${LIBICONV} ] ; then
+		tar -zxf packages/${LIBICONV}.tar.gz || { echo "failed extracting iconv "; exit 1; }
+	fi	
+
+	#---
+
+	#We also need to install nsis, though it is not a strict "dependency" per-se.
+	if [ x`which makensis` == x"" ] ; then
+		echo "Installing nsis via apt-get";
+		sudo apt-get install nsis || { echo "Failed installation"; exit 1; }
+	fi
 
-	
 	popd 2> /dev/null
 
 }
@@ -135,7 +266,7 @@ function createBaseBinaries()
 	for i in g++ cpp gcc c++
 	do
 		#Skip existing links
-		if [ -f ${HOST_VAL}-$i ] ; then
+		if [ -f ${HOST_VAL}-$i ] || [ -L ${HOST_VAL}-$i ] ; then
 			continue;
 		fi
 
@@ -203,12 +334,58 @@ function build_zlib()
 
 	#Remove the static zlib. We don't want it
 	rm $BASE/lib/libz.a
+
 	popd >/dev/null
 	popd >/dev/null
 
 	echo "zlib" >> $BUILD_STATUS_FILE
 }
 
+function build_glew()
+{
+	ISBUILT_ARG="glew"
+	isBuilt
+	if [ $ISBUILT -eq 1 ] ; then
+		return;
+	fi
+
+	#Perform dynamic modification of patch
+	if [ x`grep patches/glew-makefile.base HOST_VAL` == x""   ||  x`grep patches/glew-makefile.base BASEDIR` == x"" ] ; then
+		echo "patches/glew-makefile did not contain replacement keywords"
+		exit 1
+	fi
+
+	#Modify the patch appropriately
+	cp patches/glew-makefile.base patches/glew-makefile
+	sed -i "s@HOST_VAL@$HOST_VAL@" patches/glew-makefile
+	sed -i "s@BASEDIR@$BASE@" patches/glew-makefile
+	
+	pushd deps >/dev/null
+	pushd glew-* >/dev/null
+
+
+	if [ $? -ne 0 ] ; then
+		echo "glew dir missing, or duplicated?"
+		exit 1
+	fi
+	
+
+	APPLY_PATCH_ARG="glew-makefile"
+	applyPatches
+
+	make clean
+	rm -f configure.log
+	
+	LD=$CC make -j $NUM_PROCS || { echo "glew build failed"; exit 1; } 
+
+	make install DESTDIR="$BASE"|| { echo "glew install failed"; exit 1; } 
+
+	popd >/dev/null
+	popd >/dev/null
+
+	echo "glew" >> $BUILD_STATUS_FILE
+}
+
 function build_libpng()
 {
 	ISBUILT_ARG="libpng"
@@ -284,15 +461,29 @@ function build_libxml2()
 	fi
 
 	pushd deps >/dev/null
+	LIBXMLVER=`ls -d libxml2-* | sed 's/libxml2-//'`
 	pushd libxml2-* >/dev/null
 	
 	if [ $? -ne 0 ] ; then
 		echo "libxml2 dir missing, or duplicated?"
 		exit 1
 	fi
+
+	#Libxml 2.8 and up doesn't need patching.
+	# note that --compare-versions returns 0 on truth, and 1 on false
+	dpkg --compare-versions  $LIBXMLVER lt 2.8
+	if [ $? -eq 0 ] ; then
+		echo "WARNING : Previous attempts to use libxml2 < 2.8 have failed."
+		echo " it is recommended to manually obtain the .dsc,.orig.tar.gz and .debian.tar.gz from"
+		echo " http://packages.debian.org/source/wheezy/libxml2 . download these, then replace the .dsc,.orig.tar.gz and .debian.tar.gz in deps/packages/"
+		exit 1
+	fi
+
 	make clean
 
-	./configure --host=$HOST_VAL --enable-shared --disable-static --prefix=/ || { echo "Libxml2 configure failed"; exit 1; } 
+	#Modifications
+	#	Disable python, because sys/select.h is not in mingw
+	./configure --host=$HOST_VAL --without-python --without-html --without-http --without-ftp --without-push --without-writer --without-push --without-legacy --without-xpath --without-iconv  --enable-shared=yes --enable-static=no --prefix=/ || { echo "Libxml2 configure failed"; exit 1; } 
 
 	make -j $NUM_PROCS || { echo "libxml2 build failed"; exit 1; } 
 	
@@ -320,27 +511,28 @@ function build_libxml2()
 
 function build_libjpeg()
 {
-	NAME="libjpeg"
+		
+	NAME=$LIBJPEGNAME
 	ISBUILT_ARG=${NAME}
 	isBuilt
 	if [ $ISBUILT -eq 1 ] ; then
 		return;
 	fi
 	pushd deps >/dev/null
-	pushd libjpeg[0-9]*-* >/dev/null
+	pushd ${NAME}*[0-9]* >/dev/null
 	
 	if [ $? -ne 0 ] ; then
-		echo "libjpeg dir missing, or duplicated?"
+		echo "${NAME} dir missing, or duplicated?"
 		exit 1
 	fi
 
 	make clean
 
-	./configure --host=$HOST_VAL --enable-shared --disable-static --prefix=/ || { echo "Libjpeg configure failed"; exit 1; } 
+	./configure --host=$HOST_VAL --enable-shared --disable-static --prefix=/ || { echo "$NAME configure failed"; exit 1; } 
 
-	make -j $NUM_PROCS || { echo "libjpeg build failed"; exit 1; } 
+	make -j $NUM_PROCS || { echo "$NAME build failed"; exit 1; } 
 	
-	make install DESTDIR="$BASE"|| { echo "libjpeg install failed"; exit 1; } 
+	make install DESTDIR="$BASE"|| { echo "$NAME install failed"; exit 1; } 
 	
 	#DLL needs to be copied into lib manually
 	cp -p .libs/${NAME}-[0-9]*.dll $BASE/lib/ 
@@ -348,7 +540,7 @@ function build_libjpeg()
 	popd >/dev/null
 	popd >/dev/null
 
-	FIX_LA_FILE_ARG=libjpeg
+	FIX_LA_FILE_ARG=$NAME
 	fix_la_file
 	echo ${NAME}  >> $BUILD_STATUS_FILE
 }
@@ -362,7 +554,7 @@ function build_libtiff()
 		return;
 	fi
 	pushd deps >/dev/null
-	pushd tiff[0-9]*-* >/dev/null
+	pushd tiff*[0-9]* >/dev/null
 	
 	if [ $? -ne 0 ] ; then
 		echo "libtiff dir missing, or duplicated?"
@@ -517,6 +709,7 @@ function build_wx()
 
 	APPLY_PATCH_ARG=$PATCHES_WXWIDGETS_PRE
 	applyPatches
+#WX_DISABLE="--disable-compat26 --disable-ole --disable-dataobj --disable-ipc --disable-apple_ieee --disable-zipstream --disable-protocol_ftp --disable-mshtmlhelp --disable-aui --disable-mdi --disable-postscript --disable-datepick --disable-splash --disable-wizarddlg --disable-joystick --disable-loggui --disable-debug --disable-logwin --disable-logdlg --disable-tarstream --disable-fs_archive --disable-fs_inet --disable-fs_zip --disable-snglinst --disable-sound --disable-variant --without-regex"
 
 	./configure --host=$HOST_VAL --enable-shared --disable-static --with-opengl --enable-unicode --without-regex --prefix=/ || { echo "wxwidgets configure failed"; exit 1; } 
 
@@ -541,13 +734,13 @@ function build_wx()
 	PATCH_LEVEL=0
 	applyPatches
 	PATCH_LEVEL=1
+	sed -i "s@REPLACE_BASENAME@${BASE}@" wx-config || { echo "Failed to update wx-config with build root,. Aborting";  exit 1; }
 	popd
 
 	pushd ./lib/
 	ln -s wx-2.8/wx/ wx
 	popd
 
-	sed -i "s@REPLACE_BASENAME@${BASE}@" wx-config
 
 	echo ${NAME} >> $BUILD_STATUS_FILE
 
@@ -652,7 +845,7 @@ function build_gettext()
 	applyPatches
 	automake
 
-	./configure --host=$HOST_VAL --enable-shared --disable-static --prefix=/ || { echo "gettext configure failed"; exit 1; } 
+	./configure --host=$HOST_VAL --disable-threads --enable-shared --disable-static --prefix=/ || { echo "gettext configure failed"; exit 1; } 
 
 	make -j $NUM_PROCS || { echo "gettext build failed"; exit 1; } 
 	
@@ -686,11 +879,15 @@ function build_mathgl()
 	fi
 	make clean
 
+	APPLY_PATCH_ARG=$PATCHES_MATHGL
+	applyPatches
+
 	libtoolize --copy --force
 	aclocal
+	automake --add-missing
 
 	autoreconf
-	LIBS="${LIBS} -lz" ./configure --host=$HOST_VAL --disable-pthread --enable-shared --disable-static --prefix=/ || { echo "mathgl configure failed"; exit 1; } 
+	LIBS="${LIBS} -lz" ./configure --host=$HOST_VAL --disable-gsl --disable-pthread --enable-shared --disable-static --prefix=/ || { echo "mathgl configure failed"; exit 1; } 
 
 	#RPATH disable hack
 	sed -i 's|^hardcode_libdir_flag_spec=.*|hardcode_libdir_flag_spec=""|g' libtool
@@ -732,17 +929,34 @@ function build_ftgl()
 	fi
 	make clean
 
+	APPLY_PATCH_ARG="$PATCHES_FTGL"
+	applyPatches
 
+	autoreconf
+	libtoolize
+
+	APPLY_PATCH_ARG="$PATCHES_FTGL_POSTCONF"
+	applyPatches
 	#rename GLU to glu32 and gl to opengl32
 	sed -i s/'\-lGLU'/-lglu32/ configure
 	sed -i s/'\-lGL'/-lopengl32/ configure
 
-#	APPLY_PATCH_ARG="$PATCHES_FTGL"
-#	applyPatches
+	#configure tries to link against wrong prototypes. Override this
+	sed -i 's/char glBegin() ;//' configure
+	sed -i 's/char gluNewTess ();//' configure
+
+	sed -i 's/return glBegin ()/return 0;/' configure
+	sed -i 's/return gluNewTess ()/return 0;/' configure
+	sed -i 's/return glBegin(GL_POINTS)/return 0;/' configure
+
+
 
 	./configure --host=$HOST_VAL --enable-shared --disable-static --prefix=/ || { echo "ftgl configure failed"; exit 1; } 
 
-	
+
+	#MAkefile refers to ECHO variable for reporting completion, which does not exist
+	sed -i 's/ECHO_C =/ECHO=echo/' Makefile
+
 	make -j $NUM_PROCS || { echo "ftgl build failed"; exit 1; } 
 	
 	make install DESTDIR="$BASE"|| { echo "ftgl install failed"; exit 1; } 
@@ -822,22 +1036,87 @@ function checkHost()
 	done
 	
 	if [ x`which lsb_release` != x"" ] ; then
-		
+		#Possible results
+		# Debian, LinuxMint,Ubuntu,(others)
 		DIST_NAME=`lsb_release -i | awk -F: '{print $2}' | sed 's/\s//g'`
 		if [ x$DIST_NAME != x"Debian" ] ; then
 			echo "This is meant to target debian. I'm not sure if it will work on your system. You'll need to work that out."
+			echo "Sleeping for 4 seconds."
+			sleep 4
 		fi
 	fi
 	
 	echo "done"
 }
 
+#Build 3Depict
+function build_3Depict()
+{
+	pushd code/3Depict
+	make distclean
+
+
+	CONF_FLAG="--host=$HOST_VAL"
+	if [ $IS_RELEASE -ne 0 ] ; then
+		CONF_FLAG="$CONF_FLAG --disable-debug-checks"
+	fi
+
+	CFLAGS="$CFLAGS -DUNICODE" CPPFLAGS="${CPPFLAGS} -DUNICODE" ./configure  $CONF_FLAG
+
+	if [ $? -ne 0 ] ; then
+		echo "Failed 3Depict configure"
+		exit 1
+	fi
+
+	#HACK - strip all makefiles of -D_GLIBCXX_DEBUG
+	#	mingw & GLIBCXX_DEBG don't play nice
+	find ./ -name Makefile -exec sed -i 's/-D_GLIBCXX_DEBUG//g' {} \;
+
+	make -j$NUM_PROCS
+	if [ $? -ne 0 ] ; then
+		echo "Failed 3Depict build"
+		exit 1
+	fi
+
+	if [ $IS_RELEASE -ne 0 ] ; then
+		#Sanity check that we are actually in debug mode
+		TEST_FLAG=`./3Depict --help  2>&1 | grep "\-\-test"`
+		if [ x"$TEST_FLAG" != x"" ] ; then
+			echo "3Depict\'s Unit tests available, but should not be, when in release mode" 
+			exit 1
+		fi
+	fi
+
+
+	#if the locales are missing, try to rebuild them
+	if [ x`find locales/ -name \*.mo` = x""  ] ; then
+		
+		if [ x`which msgmerge` == x"" ] ; then
+			echo "Translations do not appear to be built for 3depict. Need to install translation builder, gettext."
+			sudo apt-get install gettext
+		fi
+		pushd translations
+		./makeTranslations || { echo "Translation build failed."; exit 1; }
+
+		mkdir -p ../locales/
+		for i in *.mo
+		do
+			TARGDIR=../locales/`echo $i | sed ' s/\.mo//' | sed 's/3Depict_//'`  
+			mkdir -p $TARGDIR
+			cp $i $TARGDIR/
+		done
+		popd
+	fi
+
+	popd > /dev/null
+}
+
 #Build the nsis package
 function make_package()
 {
 	pushd ./code/3Depict 2> /dev/null
 
-	NSI_FILE=./packaging/windows-installer/windows-installer.nsi
+	NSI_FILE=./windows-installer.nsi
 	if [ ! -f $NSI_FILE ] ; then
 		echo "NSI file missing whilst trying to build package"
 		exit 1;
@@ -847,79 +1126,98 @@ function make_package()
 	# Due to debian bug : #704828, makensis cannot correctly handle symlinks,
 	# so don't use symlinks
 	if [ ! -f `basename $NSI_FILE`  ] ; then
-		cp ./packaging/windows-installer/windows-installer.nsi .
+		cp ./packaging/mingw-debian-cross/windows-installer.nsi .
 	fi
 
 
 	echo -n " Copying dll files... "
-	DLL_FILES=`grep File windows-installer.nsi | egrep '(\.dll|\.so)' | sed 's/.*src.//' | sed 's/\"//' | sed 's/\r/ /g'`
+	SYSTEM_DLLS="(ADVAPI32.dll|COMCTL32.DLL|COMDLG32.DLL|GDI32.dll|KERNEL32.dll|ole32.dll|OLEAUT32.dll|RPCRT4.dll|SHELL32.DLL|USER32.dll|WINMM.DLL|WINSPOOL.DRV|WSOCK32.DLL|GLU32.dll|OPENGL32.dll|msvcrt.dll)"
 
+	DLL_FILES=`${HOST_VAL}-objdump -x src/3Depict.exe | grep 'DLL Name:' | awk '{print $3}' | egrep -i -v ${SYSTEM_DLLS}`
+	FOUND_DLLS=""
 	SYS_DIR=/usr/lib/gcc/${HOST_VAL}/
+
+	echo "DEBUG :" $DLL_FILES
+	rm -f tmp-dlls tmp-found-dlls
+
 	#copy the DLL files from system or 
 	# from the build locations
-	for i in ${DLL_FILES}
+	while [ x"$DLL_FILES" != x"" ] ;
 	do
-		HAVE_DLL=0
-		for j in ${BASE}/lib/ ${BASE}/bin/ $SYS_DIR
+		echo $DLL_FILES | tr '\ ' '\n' > tmp-dlls
+		DLL_FILES=`cat tmp-dlls | sort | uniq`
+
+		echo "Looking for these:" $DLL_FILES
+		echo "Have found these:" $FOUND_DLLS
+		for i in $DLL_FILES
 		do
-			FIND_RES=`find $j -name $i | head -n 1`
-			if [ x$FIND_RES != x"" ] ; then
-				HAVE_DLL=1;
-				cp $FIND_RES ./src/
-				break;
+			HAVE_DLL=0
+			for j in ${BASE}/lib/ ${BASE}/bin/ $SYS_DIR
+			do
+				FIND_RES=`find $j -name $i | head -n 1`
+				if [ x$FIND_RES != x"" ] ; then
+					HAVE_DLL=1;
+					cp $FIND_RES ./src/
+					FOUND_DLLS="$FOUND_DLLS `basename $FIND_RES`"
+					break;
+				fi
+			done
+
+			if [ $HAVE_DLL -eq 0 ] ; then
+				echo "Couldnt find DLL :" 
+				echo " $i "
+				echo " looked in ${BASE}/lib/ and $SYS_DIR"
+				exit 1;
 			fi
 		done
 
-		if [ $HAVE_DLL -eq 0 ] ; then
-			echo "Couldnt find DLL :" 
-			echo " $i "
-			echo " looked in ${BASE}/lib/ and $SYS_DIR"
-			exit 1;
-		fi
+		#Update the list of dll files
+		echo $FOUND_DLLS |tr '\ ' '\n' | sort | uniq > tmp-found-dlls
+		DLL_FILES=`${HOST_VAL}-objdump -x ./src/3Depict.exe ./src/*dll | grep 'DLL Name:' | awk '{print $3}' | egrep -i -v ${SYSTEM_DLLS} | grep -iv -f tmp-found-dlls | sort | uniq`
+
 	done
 	echo "done."
 
-	
 	if [ $IS_RELEASE -ne 0 ] ; then
-		pushd src/ 2> /dev/null
+		#Strip debugging information
+		pushd src/ > /dev/null
 		strip *.dll *.exe
-		popd 2> /dev/null
+		popd > /dev/null
 	fi
 
+	if [ x"`cat windows-installer.nsi | grep INSERT_DLLS_HERE`" == x"" ]  ||  [ x"`cat windows-installer.nsi | grep INSERT_UNINST_DLLS_HERE`" == x"" ] ; then
+		echo "DLL insertion/removal tokens not found. Was looking for INSERT_DLLS_HERE and INSERT_UNINST_DLLS_HERE"
+		exit 1
+	fi
+
+	#Insert DLL names automatically
+	cp windows-installer.nsi tmp.nsi
+	echo $FOUND_DLLS | sed 's/ /\n/g' |  sed 's@^@  File \"src\\@' | sed 's/$/\"/' > tmp-insert
+	perl -ne 's/^  ;INSERT_DLLS_HERE/`cat tmp-insert$1`/e;print' tmp.nsi >tmp2.nsi
+	mv tmp2.nsi tmp.nsi
+
+
+	echo $FOUND_DLLS | sed 's/ /\n/g' |  sed 's@^@  Delete \"$INSTDIR\\@' | sed 's/$/\"/' > tmp-insert
+	perl -ne 's/^  ;INSERT_UNINST_DLLS_HERE/`cat tmp-insert$1`/e;print' tmp.nsi > tmp2.nsi
+	mv tmp2.nsi tmp.nsi
+
+	#TODO: Why is perl converting this to dos?
+	dos2unix tmp.nsi
+
+	NSI_FILE=tmp.nsi
 
 	makensis `basename $NSI_FILE` ||  { echo "makensis failed" ; exit 1; }
+
+	if [ $IS_RELEASE -ne 0 ] ; then
+		VERSION=`cat $NSI_FILE | grep "define PRODUCT_VERSION " | awk '{print $3}' | sed s/\"//g | sed s/\s*$//`
+		TARGET_FILE=3Depict-$VERSION-$HOST_EXT.exe
+		mv Setup.exe  $TARGET_FILE
+		echo "-------------------"
+		echo "File written to : `pwd`/$TARGET_FILE"
+		echo "-------------------"
+	fi
 	
-	
-	popd
-}
-
-function make_3depict()
-{
-	cd code/3Depict
-	make distclean
-
-	if [ $IS_RELEASE -eq 1 ] ;  then
-		CONFIGURE_ARGS="--disable-debug-checks"
-	fi
-
-	CFLAGS="$CFLAGS -DUNICODE" CPPFLAGS="${CPPFLAGS} -DUNICODE" ./configure --host=$HOST_VAL $CONFIGURE_ARGS
-
-	if [ $? -ne 0 ] ; then
-		echo "Failed 3Depict configure"
-		exit 1
-	fi
-
-	#HACK - strip all makefiles of -D_GLIBCXX_DEBUG, even when not in release mode
-	# as a workaround for broken configure not handling debian bug 703935
-	if [ $IS_RELEASE -ne 1 ] ;  then
-		find ./ -name Makefile -exec sed -i 's/-D_GLIBCXX_DEBUG//g' {} \;
-	fi
-
-	make -j$NUM_PROCS
-	if [ $? -ne 0 ] ; then
-		echo "Failed 3Depict build"
-		exit 1
-	fi
+	popd > /dev/null
 }
 
 #Check that we have a suitable host system
@@ -931,28 +1229,39 @@ checkPatchesExist
 #build the dirs we need
 createDirLayout
 
+
+#Install cross compiler
+#---
+case ${HOST_VAL}  in
+	x86_64-w64-mingw32)
+		if [ x"$DIST_NAME" == x"Ubuntu" ] || [ x"$DIST_NAME" == x"LinuxMint" ] ; then
+			MINGW_PACKAGES="mingw-w64-dev g++-mingw-w64-x86-64"
+		else
+			MINGW_PACKAGES="mingw-w64-x86-64-dev g++-mingw-w64-x86-64"
+		fi
+		HOST_EXT="win64"
+	;;
+	i686-w64-mingw32)
+		MINGW_PACKAGES="gcc-mingw32"
+		HOST_EXT="win32"
+	;;
+	*)
+		echo "Unknown host... please install deps manually,or alter script"
+		exit 1	
+	;;
+esac
+
+#install the compiler
+install_mingw
+#---
+
 #Create the binaries we need
 createBaseBinaries
 
 #Obtain the needed dependencies
 grabDeps
 
-#Install cross compiler
-case echo ${HOST_VAL} 
-	
-	x86_64-w64-mingw32)
-		MINGW_PACKAGES="gcc-mingw32"
-	;;
-	HOST_VAL=i686-w64-mingw32)
-		MINGW_PACKAGES="mingw-w64-x86-64-dev g++-mingw-w64-x86-64"
-	;;
-	*)
-	echo "Unknown host... please install deps manually,or alter script"
-	return
-	;;
-esac
 
-install_mingw
 
 #set our needed environment variables
 PATH=${BASE}/bin/:/usr/$HOST_VAL/bin/:$PATH
@@ -975,14 +1284,15 @@ build_libxml2
 build_gsl
 build_qhull
 build_expat
-build_wx	# I'm not sure I've done this 100% right. Check wx-config output 
 build_freetype
 build_libiconv
 build_gettext 
 build_mathgl 
 build_ftgl 
+build_glew
+build_wx	# I'm not sure I've done this 100% right. Check wx-config output 
 
-
+build_3Depict
 
 make_package
 
