@@ -17,7 +17,7 @@
 */
 
 #include "configFile.h"
-#include "wxcommon.h"
+#include "wx/wxcommon.h"
 
 
 #include "backend/filters/allFilter.h"
@@ -43,10 +43,10 @@ using std::string;
 
 
 ConfigFile::ConfigFile() : configLoadOK(false), panelMode(CONFIG_PANELMODE_REMEMBER),
-	haveIntialAppSize(false), mouseZoomRatePercent(100),mouseMoveRatePercent(100),
-	allowOnline(true), allowOnlineVerCheck(true), leftRightSashPos(0),
-	topBottomSashPos(0),filterSashPos(0),plotListSashPos(0)
-
+	haveInitialAppSize(false), mouseZoomRatePercent(100),mouseMoveRatePercent(100),
+	wantStartupOrthoCam(false),allowOnline(true), allowOnlineVerCheck(true), leftRightSashPos(0),
+	topBottomSashPos(0),filterSashPos(0),plotListSashPos(0), haveMaxPoints(false),
+	maxPointsScene(0), doWantStartupTips(true)
 { 
 }
 
@@ -107,18 +107,18 @@ void ConfigFile::setFilterDefaults(const vector<Filter *>  &defs)
 
 bool ConfigFile::getInitialAppSize(unsigned int &x, unsigned int &y) const
 {
-	if(haveIntialAppSize)
+	if(haveInitialAppSize)
 	{
 		x=initialSizeX;
 		y=initialSizeY;
 	}
 	
-	return haveIntialAppSize;
+	return haveInitialAppSize;
 }
 
 void ConfigFile::setInitialAppSize(unsigned int x, unsigned int y)
 {
-	haveIntialAppSize=true;
+	haveInitialAppSize=true;
 	initialSizeX=x;
 	initialSizeY=y;
 }
@@ -157,7 +157,7 @@ unsigned int ConfigFile::read()
 	}
 
 	//Open the XML file again, but without DTD validation
-	doc = xmlCtxtReadFile(context, filename.c_str(), NULL, 0);
+	doc = xmlCtxtReadFile(context, filename.c_str(), NULL, XML_PARSE_NONET|XML_PARSE_NOENT);
 
 	if(!doc)
 		return CONFIG_ERR_NOFILE;
@@ -199,7 +199,7 @@ unsigned int ConfigFile::read()
 				XMLGetAttrib(nodePtr,initialSizeY,"height"))
 			{
 				if( initialSizeX >0 && initialSizeY > 0)
-					haveIntialAppSize=true;
+					haveInitialAppSize=true;
 			}
 		}
 		nodePtr=nodeStack.top();
@@ -461,6 +461,43 @@ unsigned int ConfigFile::read()
 		}
 		nodePtr=nodeStack.top();
 		nodeStack.pop();
+
+
+		nodeStack.push(nodePtr);
+		haveMaxPoints=XMLGetNextElemAttrib(nodePtr,maxPointsScene,"maxdisplaypoints","value");
+
+		nodePtr=nodeStack.top();
+		nodeStack.pop();
+
+		nodeStack.push(nodePtr);
+		//have we seen a startup tip entry?
+		if(!XMLHelpFwdToElem(nodePtr,"startuptips"))
+		{
+			std::string str;
+			XMLGetAttrib(nodePtr,str,"value");
+
+			//Check if the user wants startup tips. If we cant understand this, then say no.
+			if(!boolStrDec(str,doWantStartupTips))
+				doWantStartupTips=false;
+		}
+		nodePtr=nodeStack.top();
+		nodeStack.pop();
+		
+		nodeStack.push(nodePtr);
+		//Does the user want, by default, an orthographic camera
+		if(!XMLHelpFwdToElem(nodePtr,"wantorthocam"))
+		{
+			std::string str;
+			XMLGetAttrib(nodePtr,str,"value");
+
+			//Check if the user wants ortho camera, if no idea, revert to not 
+			if(!boolStrDec(str,wantStartupOrthoCam))
+				wantStartupOrthoCam=false;
+		}
+		nodePtr=nodeStack.top();
+		nodeStack.pop();
+
+
 nodeptrEndJump:
 		;
 
@@ -501,11 +538,14 @@ bool ConfigFile::createConfigDir()
 
 std::string ConfigFile::getConfigDir() 
 {
- 	wxStandardPaths *paths = new wxStandardPaths;
+#if wxCHECK_VERSION(2,9,0)
+ 	wxStandardPaths &paths = wxStandardPaths::Get();
+	wxString filePath = paths.GetDocumentsDir()+wxCStr("/.")+wxCStr(PROGRAM_NAME);
+#else
+	wxStandardPaths *paths = new wxStandardPaths;
 	wxString filePath = paths->GetDocumentsDir()+wxCStr("/.")+wxCStr(PROGRAM_NAME);
-	
 	delete paths;
-	
+#endif
 	return stlStr(filePath);
 }
 
@@ -529,7 +569,7 @@ bool ConfigFile::write()
 	f<< "<threeDepictconfig>" << endl;
 	f<<tabs(1)<< "<writer version=\"" << PROGRAM_VERSION << "\"/>" << endl;
 
-	if(haveIntialAppSize)	
+	if(haveInitialAppSize)	
 	{
 		f<<tabs(1)<< "<initialwinsize width=\"" << initialSizeX << "\" height=\"" <<
 				initialSizeY << "\"/>" << endl;
@@ -587,6 +627,13 @@ bool ConfigFile::write()
 			f << tabs(2) << "<pos name=\"plotlist\" value=\"" << plotListSashPos<< "\"/>" << endl;
 	f << tabs(1) << "</sashposition>" << endl;
 
+
+	if(haveMaxPoints)
+		f << tabs(1) << "<maxdisplaypoints value=\"" << maxPointsScene << "\"/>" << endl;
+
+	f << tabs(1) << "<startuptips value=\"" << boolStrEnc(doWantStartupTips) << "\"/>" <<endl;
+	f << tabs(1) << "<wantorthocam value=\"" << boolStrEnc(wantStartupOrthoCam) << "\"/>" <<endl;
+
 	f << "</threeDepictconfig>" << endl;
 
 	ASSERT(isValidXML(filename.c_str()));
