@@ -17,15 +17,20 @@
 */
 #ifndef FILTER_H
 #define FILTER_H
+
 class Filter;
 class FilterStreamData;
+
 class ProgressData;
 class RangeFileFilter;
 
+#include "APT/ionhit.h"
+
 #include "APT/APTRanges.h"
 #include "common/constants.h"
 
 #include "gl/select.h"
+#include "gl/drawables.h"
 
 #include "common/voxels.h"
 
@@ -75,7 +80,7 @@ extern const char *FILTER_NAMES[];
 //Current bitmask using functions are
 //	VisController::safeDeleteFilterList
 const unsigned int NUM_STREAM_TYPES=5;
-const unsigned int STREAMTYPE_MASK_ALL= (1<<(NUM_STREAM_TYPES)) -1;
+const unsigned int STREAMTYPE_MASK_ALL= ((1<<(NUM_STREAM_TYPES)) -1 ) & 0x000000FF;
 enum
 {
 	STREAM_TYPE_IONS=1,
@@ -116,6 +121,7 @@ enum
 	//VoxelStreamData
 	VOXEL_REPRESENT_POINTCLOUD,
 	VOXEL_REPRESENT_ISOSURF,
+	VOXEL_REPRESENT_AXIAL_SLICE,
 	VOXEL_REPRESENT_END
 };
 
@@ -131,18 +137,24 @@ enum
 	FILE_TYPE_POS
 };
 
+
+enum
+{
+	FILTER_ERR_ABORT=1, //User abort
+	FILTER_ERR_ENUM_END
+};
+
 //---
 //
 
 //Forward dec.
-class FilterStreamData;
-class wxPropertyGrid;
+class wxCustomPropGrid;
 
 //!Return the number of elements in a vector of filter data - i.e. the sum of the number of objects within each stream. Only masked streams (STREAM_TYPE_*) will be counted
 size_t numElements(const vector<const FilterStreamData *> &vm, unsigned int mask=STREAMTYPE_MASK_ALL);
 
 
-void updateFilterPropertyGrid(wxPropertyGrid *g, const Filter *f);
+void updateFilterPropertyGrid(wxCustomPropGrid *g, const Filter *f);
 
 //!Abstract base class for data types that can propagate through filter system
 class FilterStreamData
@@ -153,7 +165,7 @@ class FilterStreamData
 		//!Parent filter pointer
 		const Filter *parent;
 
-		//!Tells us if the filter has cacehd this data for later use. 
+		//!Tells us if the filter has cached this data for later use. 
 		//this is a boolean value, but not declared as such, as there 
 		//are debug traps to tell us if this is not set by looking for non-boolean values.
 		unsigned int cached;
@@ -161,7 +173,7 @@ class FilterStreamData
 		FilterStreamData();
 		virtual ~FilterStreamData() {}; 
 		virtual size_t getNumBasicObjects() const =0;
-		//!Returns an integer unique to the clas to identify type (yes rttid...)
+		//!Returns an integer unique to the class to identify type (yes rttid...)
 		virtual unsigned int getStreamType() const {return streamType;} ;
 		//!Free mem held by objects
 		virtual void clear()=0;
@@ -246,7 +258,16 @@ class IonStreamData : public FilterStreamData
 public:
 	IonStreamData();
 	void clear();
-	size_t getNumBasicObjects() const  { return data.size();};
+
+	//Sample the data vector to the specified fraction
+	void sample(float fraction);
+
+	//Duplicate this object, but only using a sampling of the data
+	// vector. The retuned object must be deleted by the
+	// caller. Cached status is *not* duplicated
+	IonStreamData *cloneSampled(float fraction) const;
+
+	size_t getNumBasicObjects() const;
 	
 	unsigned int representationType;
 	float r,g,b,a;
@@ -316,7 +337,7 @@ class PlotStreamData : public FilterStreamData
 		//!Region colours
 		vector<float> regionR,regionB,regionG;
 
-		//!Region indicies from parent region
+		//!Region indices from parent region
 		vector<unsigned int> regionID;
 
 		//!Region parent filter pointer, used for matching interaction 
@@ -351,7 +372,7 @@ class DrawStreamData: public FilterStreamData
 		//!Returns 0, as this does not store basic object types -- i.e. is not for data storage per se.
 		size_t getNumBasicObjects() const { return 0; }
 
-		//!Erase the drawing vector, deleting its componets
+		//!Erase the drawing vector, deleting its components
 		void clear();
 #ifdef DEBUG
 		//Cross-checks fields to determine if (best guess)
@@ -398,10 +419,9 @@ class Filter
 		bool cache, cacheOK;
 		static bool strongRandom;
 
-		bool wantMonitor;
 
 		//!Array of the number of streams propagated on last refresh
-		//THis is initialised to -1, which is considered invalid
+		//This is initialised to -1, which is considered invalid
 		unsigned int numStreamsLastRefresh[NUM_STREAM_TYPES];
 	
 
@@ -412,7 +432,22 @@ class Filter
 		//Filter output cache
 		std::vector<FilterStreamData *> filterOutputs;
 		//!User interaction "Devices" associated with this filter
-		std::vector<SelectionDevice<Filter> *> devices;
+		std::vector<SelectionDevice *> devices;
+
+
+
+		//Collate ions from filterstream data into an ionhit vector
+		static unsigned int collateIons(const vector<const FilterStreamData *> &dataIn,
+				vector<IonHit> &outVector, ProgressData &prog,
+				bool (*callback)(bool),size_t totalDataSize=(size_t)-1);
+
+		//!Propagate the given input data to an output vector
+		void propagateStreams(const vector<const FilterStreamData *> &dataIn,
+				vector<const FilterStreamData *> &dataOut,size_t mask=STREAMTYPE_MASK_ALL,bool invertMask=false) const;
+
+		//!Propagate the cache into output
+		void propagateCache(vector<const FilterStreamData *> &dataOut) const;
+
 	public:	
 		Filter() ;
 		virtual ~Filter();
@@ -446,7 +481,7 @@ class Filter
 
 		//!Set the properties for the nth filter, 
 		//!needUpdate tells us if filter output changes due to property set
-		//NOte that if you modify a result without clearing the cache,
+		//Note that if you modify a result without clearing the cache,
 		//then any downstream decision based upon that may not be noted in an update
 		//Take care.
 		virtual bool setProperty(unsigned int key,
@@ -476,8 +511,9 @@ class Filter
 		virtual unsigned int getRefreshEmitMask() const = 0;
 
 
-		//!Mask of filter streams that will not examined by the filter in its computation.
-		// note that these *may* be emitted as a pass-through - check emitmask if needed
+		//!Mask of filter streams that will be examined by the filter in its computation.
+		// note that output may be emitted as a pass-through even if the use is not set
+		// - check emitmask if needed
 		virtual unsigned int getRefreshUseMask() const =0;
 
 		//====
@@ -522,7 +558,7 @@ class Filter
 		 * another at this level (for example setting two devices on one primitve,
 		 * with the same mouse/key bindings). So dont do that.
 		 */
-		void getSelectionDevices(vector<SelectionDevice<Filter> *> &devices);
+		void getSelectionDevices(vector<SelectionDevice *> &devices) const;
 
 
 		//!Update the output statistics for this filter (num items of streams of each type output)
@@ -540,9 +576,9 @@ class Filter
 		//!Get the number of outputs for the specified type during the filter's last refresh
 		unsigned int getNumOutput(unsigned int streamType) const;
 
-		//!Get the filter messages from the console -- also deletes the 
-		// console messages as a side-effect
-		void getConsoleStrings(std::vector<std::string > &v) { v.resize(consoleOutput.size());std::copy(consoleOutput.begin(),consoleOutput.end(),v.begin()); consoleOutput.clear();};
+		//!Get the filter messages from the console. To erase strings, either call erase, or erase cahche
+		void getConsoleStrings(std::vector<std::string > &v) const { v=consoleOutput;};
+		void clearConsole() { consoleOutput.clear();};
 
 		//!Should filters use strong randomisation (where applicable) or not?
 		static void setStrongRandom(bool strongRand) {strongRandom=strongRand;}; 
@@ -575,7 +611,7 @@ class ProgressData
 	public:
 		//!Progress of filter (out of 100) for current filter
 		unsigned int filterProgress;
-		//!Number of filters (n) that we have proccessed (n out of m filters)
+		//!Number of filters (n) that we have processed (n out of m filters)
 		unsigned int totalProgress;
 
 		//!number of filters which need processing for this update
@@ -592,6 +628,9 @@ class ProgressData
 		//!Name of current operation, if specified
 		std::string stepName;
 
+		bool operator==(const ProgressData &o) const;
+		const ProgressData &operator=(const ProgressData &o);
+
 		void reset() { filterProgress=totalProgress=step=maxStep=0;curFilter=0; stepName.clear();};
 		void clock() { filterProgress=step=maxStep=0;curFilter=0;totalProgress++; stepName.clear();};
 };
