@@ -366,9 +366,7 @@ unsigned int BoundingBoxFilter::refresh(const std::vector<const FilterStreamData
 
 	if(!isVisible)
 	{
-		for(unsigned int ui=0;ui<dataIn.size();ui++)
-			getOut.push_back(dataIn[ui]);
-
+		propagateStreams(dataIn,getOut);
 		return 0;
 	}
 
@@ -1147,7 +1145,7 @@ bool boxVolumeTest()
 	}
 
 
-	TEST(havePrismDrawable, "bounding box existance test");
+	TEST(havePrismDrawable, "bounding box existence test");
 	
 	TEST(fabs(bc.volume() - 1.0f) 
 		< sqrt(std::numeric_limits<float>::epsilon()),
