@@ -147,6 +147,7 @@ unsigned int CompositionProfileFilter::refresh(const std::vector<const FilterStr
 								bool (*callback)(bool))
 {
 	//Clear selection devices
+	// FIXME: Leaking drawables.
 	clearDevices();
 	
 	if(showPrimitive)
@@ -181,7 +182,7 @@ unsigned int CompositionProfileFilter::refresh(const std::vector<const FilterStr
 				//tapered cylinder)
 				dC->lockRadii();
 
-				SelectionDevice<Filter> *s = new SelectionDevice<Filter>(this);
+				SelectionDevice *s = new SelectionDevice(this);
 				SelectionBinding b;
 				//Bind the drawable object to the properties we wish
 				//to be able to modify
@@ -250,7 +251,7 @@ unsigned int CompositionProfileFilter::refresh(const std::vector<const FilterStr
 				//The object is selectable
 				dS->canSelect=true;
 
-				SelectionDevice<Filter> *s = new SelectionDevice<Filter>(this);
+				SelectionDevice *s = new SelectionDevice(this);
 				SelectionBinding b[3];
 
 				//Apple doesn't have right click, so we need
@@ -291,17 +292,12 @@ unsigned int CompositionProfileFilter::refresh(const std::vector<const FilterStr
 	if(cacheOK)
 	{
 		//propagate our cached plot data.
-		for(unsigned int ui=0;ui<filterOutputs.size();ui++)
-			getOut.push_back(filterOutputs[ui]);
+		propagateCache(getOut);
 
 		ASSERT(filterOutputs.back()->getStreamType() == STREAM_TYPE_PLOT);
 
-		//Propagate all the incoming data (including ions)
-		for(unsigned int ui=0;ui<dataIn.size() ;ui++)
-		{
-			if(dataIn[ui]->getStreamType() != STREAM_TYPE_IONS)
-				getOut.push_back(dataIn[ui]);
-		}
+		//Propagate all the incoming data (excluding ions)
+		propagateStreams(dataIn,getOut,STREAM_TYPE_IONS,true);
 			
 		return 0;
 	}
@@ -322,6 +318,10 @@ unsigned int CompositionProfileFilter::refresh(const std::vector<const FilterStr
 	float length;
 	unsigned int numBins, errCode;
 	errCode=getBinData(numBins,length);
+
+	if(!numBins)
+		return 0;
+
 	if(errCode)
 		return errCode;
 
@@ -456,7 +456,7 @@ unsigned int CompositionProfileFilter::refresh(const std::vector<const FilterStr
 			if(rngData)
 				plotData[ui]->yLabel= TRANS("Fraction");
 			else
-				plotData[ui]->yLabel= TRANS("Density (\\#.len^3)");
+				plotData[ui]->yLabel= TRANS("Density (\\frac{\\#}{len^3})");
 		}
 		else
 			plotData[ui]->yLabel= TRANS("Count");
@@ -648,7 +648,9 @@ bool CompositionProfileFilter::setProperty( unsigned int key,
 					newPt*=sqrt(vectorParams[1].sqrMag());
 				}
 			}
-			
+			if(newPt.sqrMag() < sqrt(std::numeric_limits<float>::epsilon()))
+				return false;
+
 			if(!(vectorParams[1] == newPt ))
 			{
 				vectorParams[1] = newPt;
@@ -762,6 +764,9 @@ bool CompositionProfileFilter::setProperty( unsigned int key,
 			if(stream_cast(newRad,value))
 				return false;
 
+			if(newRad < sqrt(std::numeric_limits<float>::epsilon()))
+				return false;
+
 			if(scalarParams[0] != newRad )
 			{
 				scalarParams[0] = newRad;
@@ -1285,7 +1290,7 @@ bool CompositionProfileFilter::readState(xmlNodePtr &nodePtr, const std::string
 	xmlFree(xmlString);
 	//====
 	
-	//Retreive vector parameters
+	//Retrieve vector parameters
 	//===
 	if(XMLHelpFwdToElem(nodePtr,"vectorparams"))
 		return false;
@@ -1335,7 +1340,7 @@ bool CompositionProfileFilter::readState(xmlNodePtr &nodePtr, const std::string
 	//===	
 
 	nodePtr=tmpNode;
-	//Retreive scalar parameters
+	//Retrieve scalar parameters
 	//===
 	if(XMLHelpFwdToElem(nodePtr,"scalarparams"))
 		return false;
@@ -1515,8 +1520,22 @@ void CompositionProfileFilter::setPropFromBinding(const SelectionBinding &b)
 			b.getValue(vectorParams[0]);
 			break;
 		case BINDING_CYLINDER_DIRECTION:
+		{
+			Point3D pOld=vectorParams[1];
 			b.getValue(vectorParams[1]);
+			//Test getting the bin data.
+			// if something is wrong, abort
+			float length;
+			unsigned int numBins;
+			unsigned int errCode= getBinData(numBins,length);
+			if(errCode || !numBins)
+			{
+				vectorParams[1]=pOld;
+				return;
+			}
+
 			break;
+		}
 		default:
 			ASSERT(false);
 	}
@@ -1524,7 +1543,7 @@ void CompositionProfileFilter::setPropFromBinding(const SelectionBinding &b)
 	clearCache();
 }
 
-unsigned int CompositionProfileFilter::getPrimitiveId(const std::string &primitiveName) const
+unsigned int CompositionProfileFilter::getPrimitiveId(const std::string &primitiveName) 
 {
 	for(size_t ui=0;ui<PRIMITIVE_END; ui++)
 	{
