@@ -16,20 +16,15 @@
  *	along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 #include "externalProgram.h"
-#include "../../wxcommon.h"
 
 #include "filterCommon.h"
 
+#include "../../wx/wxcommon.h"
+#include "backend/APT/APTFileIO.h"
+
 #include <wx/filename.h>
 #include <wx/dir.h>
 
-enum
-{
-	KEY_COMMAND,
-	KEY_WORKDIR,
-	KEY_ALWAYSCACHE,
-	KEY_CLEANUPINPUT
-};
 
 //!Error codes
 enum
@@ -80,114 +75,16 @@ size_t ExternalProgramFilter::numBytesForCache(size_t nObjects) const
 		return (size_t)-1; //Say we don't know, we are not going to cache anyway.
 }
 
-unsigned int ExternalProgramFilter::refresh(const std::vector<const FilterStreamData *> &dataIn,
-	std::vector<const FilterStreamData *> &getOut, ProgressData &progress, bool (*callback)(bool))
+size_t ExternalProgramFilter::substituteVariables(const std::string &commandStr,
+			const vector<string> &ionOutputNames, const vector<string> &plotOutputNames,std::string &substitutedCommand) 
 {
-	//use the cached copy if we have it.
-	if(cacheOK)
-	{
-		for(unsigned int ui=0;ui<filterOutputs.size();ui++)
-			getOut.push_back(filterOutputs[ui]);
-
-		return 0;
-	}
-
 	vector<string> commandLineSplit;
 
-	splitStrsRef(commandLine.c_str(),' ',commandLineSplit);
+	splitStrsRef(commandStr.c_str(),' ',commandLineSplit);
 	//Nothing to do
 	if(commandLineSplit.empty())
 		return 0;
 
-	vector<string> ionOutputNames,plotOutputNames;
-
-	//Compute the bounding box of the incoming streams
-	string s;
-	wxString tempDir;
-	if(workingDir.size())
-		tempDir=(wxStr(workingDir) +wxT("/inputData"));
-	else
-		tempDir=(wxT("inputData"));
-
-
-	//Create a temporary dir
-	if(!wxDirExists(tempDir) )
-	{
-		//Audacity claims that this can return false even on
-		//success (NoiseRemoval.cpp, line 148).
-		//I was having problems with this function too;
-		//so use their workaround
-		wxMkdir(tempDir);
-
-		if(!wxDirExists(tempDir) )	
-			return MAKEDIR_FAIL;
-
-	}
-	
-	for(unsigned int ui=0;ui<dataIn.size() ;ui++)
-	{
-		switch(dataIn[ui]->getStreamType())
-		{
-			case STREAM_TYPE_IONS:
-			{
-				const IonStreamData *i;
-				i = (const IonStreamData * )(dataIn[ui]);
-
-				if(i->data.empty())
-					break;
-				//Save the data to a file
-				wxString tmpStr;
-
-				tmpStr=wxFileName::CreateTempFileName(tempDir+ wxT("/pointdata"));
-				//wxwidgets has no suffix option... annoying.
-				wxRemoveFile(tmpStr);
-
-				s = stlStr(tmpStr);
-				s+=".pos";
-				if(IonVectorToPos(i->data,s))
-				{
-					//Uh-oh problem. Clean up and exit
-					return WRITEPOS_FAIL;
-				}
-
-				ionOutputNames.push_back(s);
-				break;
-			}
-			case STREAM_TYPE_PLOT:
-			{
-				const PlotStreamData *i;
-				i = (const PlotStreamData * )(dataIn[ui]);
-
-				if(i->xyData.empty())
-					break;
-				//Save the data to a file
-				wxString tmpStr;
-
-				tmpStr=wxFileName::CreateTempFileName(tempDir + wxT("/plot"));
-				//wxwidgets has no suffix option... annoying.
-				wxRemoveFile(tmpStr);
-				s = stlStr(tmpStr);
-			        s+= ".xy";
-				if(!writeTextFile(s.c_str(),i->xyData))
-				{
-					//Uh-oh problem. Clean up and exit
-					return WRITEPLOT_FAIL;
-				}
-
-				plotOutputNames.push_back(s);
-				break;
-			}
-			default:
-				break;
-		}	
-	}
-
-	//Nothing to do.
-	if(plotOutputNames.empty() &&
-		ionOutputNames.empty())
-		return 0;
-
-
 	//Construct the command, using substitution
 	string command;
 	unsigned int ionOutputPos,plotOutputPos;
@@ -201,8 +98,8 @@ unsigned int ExternalProgramFilter::refresh(const std::vector<const FilterStream
 		}
 		else
 		{
-			size_t pos;
-			pos=0;
+			size_t pos,lastPos;
+			lastPos=pos=0;
 
 			string thisCommandEntry;
 			pos =commandLineSplit[ui].find("%",pos);
@@ -218,6 +115,8 @@ unsigned int ExternalProgramFilter::refresh(const std::vector<const FilterStream
 				char code;
 				code = commandLineSplit[ui][pos+1];
 
+				thisCommandEntry+=commandLineSplit[ui].substr(lastPos,pos-lastPos);
+
 				switch(code)
 				{
 					case '%':
@@ -290,14 +189,132 @@ unsigned int ExternalProgramFilter::refresh(const std::vector<const FilterStream
 				}
 
 
+				lastPos=pos;
 				pos =commandLineSplit[ui].find("%",pos);
 			}
-			
+		
+			thisCommandEntry+=commandLineSplit[ui].substr(lastPos+1);
+
 			command+= string(" ") + thisCommandEntry;
 		}
 
 	}
 
+
+	command.swap(substitutedCommand);
+
+	return 0;
+}
+
+unsigned int ExternalProgramFilter::refresh(const std::vector<const FilterStreamData *> &dataIn,
+	std::vector<const FilterStreamData *> &getOut, ProgressData &progress, bool (*callback)(bool))
+{
+	//use the cached copy if we have it.
+	if(cacheOK)
+	{
+		propagateCache(getOut);
+
+		return 0;
+	}
+
+	if(commandLine.empty())
+		return 0;
+
+	vector<string> ionOutputNames,plotOutputNames;
+
+	//Compute the bounding box of the incoming streams
+	string s;
+	wxString tempDir;
+	if(workingDir.size())
+		tempDir=(wxStr(workingDir) +wxT("/inputData"));
+	else
+		tempDir=(wxT("inputData"));
+
+
+	//Create a temporary dir
+	if(!wxDirExists(tempDir) )
+	{
+		//Audacity claims that this can return false even on
+		//success (NoiseRemoval.cpp, line 148).
+		//I was having problems with this function too;
+		//so use their workaround
+		wxMkdir(tempDir);
+
+		if(!wxDirExists(tempDir) )	
+			return MAKEDIR_FAIL;
+
+	}
+	
+	for(unsigned int ui=0;ui<dataIn.size() ;ui++)
+	{
+		switch(dataIn[ui]->getStreamType())
+		{
+			case STREAM_TYPE_IONS:
+			{
+				const IonStreamData *i;
+				i = (const IonStreamData * )(dataIn[ui]);
+
+				if(i->data.empty())
+					break;
+				//Save the data to a file
+				wxString tmpStr;
+
+				tmpStr=wxFileName::CreateTempFileName(tempDir+ wxT("/pointdata"));
+				//wxwidgets has no suffix option... annoying.
+				wxRemoveFile(tmpStr);
+
+				s = stlStr(tmpStr);
+				s+=".pos";
+				if(IonHit::makePos(i->data,s.c_str()))
+				{
+					//Uh-oh problem. Clean up and exit
+					return WRITEPOS_FAIL;
+				}
+
+				ionOutputNames.push_back(s);
+				break;
+			}
+			case STREAM_TYPE_PLOT:
+			{
+				const PlotStreamData *i;
+				i = (const PlotStreamData * )(dataIn[ui]);
+
+				if(i->xyData.empty())
+					break;
+				//Save the data to a file
+				wxString tmpStr;
+
+				tmpStr=wxFileName::CreateTempFileName(tempDir + wxT("/plot"));
+				//wxwidgets has no suffix option... annoying.
+				wxRemoveFile(tmpStr);
+				s = stlStr(tmpStr);
+			        s+= ".xy";
+				if(!writeTextFile(s.c_str(),i->xyData))
+				{
+					//Uh-oh problem. Clean up and exit
+					return WRITEPLOT_FAIL;
+				}
+
+				plotOutputNames.push_back(s);
+				break;
+			}
+			default:
+				break;
+		}	
+	}
+
+	//Nothing to do.
+	if(plotOutputNames.empty() &&
+		ionOutputNames.empty())
+		return 0;
+
+	std::string substitutedCommand;
+	size_t errCode;
+	errCode=substituteVariables(commandLine,ionOutputNames,plotOutputNames,
+					substitutedCommand);
+	if(errCode)
+		return errCode;
+
 	//If we have a specific working dir; use it. Otherwise just use curdir
 	wxString origDir = wxGetCwd();
 	if(workingDir.size())
@@ -308,13 +325,13 @@ unsigned int ExternalProgramFilter::refresh(const std::vector<const FilterStream
 	}
 	else
 	{
-		if(!wxSetWorkingDirectory(_(".")))
+		if(!wxSetWorkingDirectory(wxT(".")))
 			return SETWORKDIR_FAIL;
 	}
 
 	bool result;
 	//Execute the program
-	result=wxShell(wxStr(command));
+	result=wxShell(wxStr(substitutedCommand));
 
 	if(cleanInput)
 	{
@@ -346,9 +363,9 @@ unsigned int ExternalProgramFilter::refresh(const std::vector<const FilterStream
 	wxDir *dir = new wxDir;
 	wxArrayString *a = new wxArrayString;
 	if(workingDir.size())
-		dir->GetAllFiles(wxStr(workingDir),a,_("*.pos"),wxDIR_FILES);
+		dir->GetAllFiles(wxStr(workingDir),a,wxT("*.pos"),wxDIR_FILES);
 	else
-		dir->GetAllFiles(wxGetCwd(),a,_("*.pos"),wxDIR_FILES);
+		dir->GetAllFiles(wxGetCwd(),a,wxT("*.pos"),wxDIR_FILES);
 
 
 	//read the output files, which is assumed to be any "pos" file
@@ -397,9 +414,9 @@ unsigned int ExternalProgramFilter::refresh(const std::vector<const FilterStream
 
 	a->Clear();
 	if(workingDir.size())
-		dir->GetAllFiles(wxStr(workingDir),a,_("*.xy"),wxDIR_FILES);
+		dir->GetAllFiles(wxStr(workingDir),a,wxT("*.xy"),wxDIR_FILES);
 	else
-		dir->GetAllFiles(wxGetCwd(),a,_("*.xy"),wxDIR_FILES);
+		dir->GetAllFiles(wxGetCwd(),a,wxT("*.xy"),wxDIR_FILES);
 
 	//read the output files, which is assumed to be any "pos" file
 	//in the working dir
@@ -422,7 +439,7 @@ unsigned int ExternalProgramFilter::refresh(const std::vector<const FilterStream
 			//Possible delimiters to try when loading file
 			//try each in turn
 			const char *delimString ="\t, ";
-			if(!loadTextData(sTmp.c_str(),dataVec,header,delimString))
+			if(loadTextData(sTmp.c_str(),dataVec,header,delimString))
 				return READPLOT_FAIL;
 
 			//Check that the input has the correct size
@@ -449,6 +466,9 @@ unsigned int ExternalProgramFilter::refresh(const std::vector<const FilterStream
 				d->g=1.0;
 				d->b=0;
 				d->a=1.0;
+				d->index=uj;
+				d->plotMode=PLOT_MODE_1D;
+				d->plotStyle=PLOT_TRACE_LINES;
 
 
 				//set the title to the filename (trim the .xy extension
@@ -514,14 +534,14 @@ void ExternalProgramFilter::getProperties(FilterPropGroup &propertyList) const
 	p.data= commandLine;
 	p.type=PROPERTY_TYPE_STRING;
 	p.helpText=TRANS("Full command to send to operating system. See manual for escape sequence meanings");
-	p.key=KEY_COMMAND;
+	p.key=EXTERNALPROGRAM_KEY_COMMAND;
 	propertyList.addProperty(p,curGroup);
 	
 	p.name=TRANS("Work Dir");
 	p.data= workingDir;
 	p.type=PROPERTY_TYPE_STRING;
 	p.helpText=TRANS("Directory to run the command in");
-	p.key=KEY_WORKDIR;		
+	p.key=EXTERNALPROGRAM_KEY_WORKDIR;		
 	propertyList.addProperty(p,curGroup);
 	
 
@@ -534,7 +554,7 @@ void ExternalProgramFilter::getProperties(FilterPropGroup &propertyList) const
 	p.data=tmpStr;
 	p.type=PROPERTY_TYPE_BOOL;
 	p.helpText=TRANS("Erase input files when command completed");
-	p.key=KEY_CLEANUPINPUT;		
+	p.key=EXTERNALPROGRAM_KEY_CLEANUPINPUT;		
 	propertyList.addProperty(p,curGroup);
 	
 	if(alwaysCache)
@@ -546,7 +566,7 @@ void ExternalProgramFilter::getProperties(FilterPropGroup &propertyList) const
 	p.data=tmpStr;
 	p.type=PROPERTY_TYPE_BOOL;
 	p.helpText=TRANS("Assume program does not alter its output, unless inputs from 3Depict are altered");
-	p.key=KEY_ALWAYSCACHE;		
+	p.key=EXTERNALPROGRAM_KEY_ALWAYSCACHE;		
 	propertyList.addProperty(p,curGroup);
 
 }
@@ -557,7 +577,7 @@ bool ExternalProgramFilter::setProperty(  unsigned int key,
 	needUpdate=false;
 	switch(key)
 	{
-		case KEY_COMMAND:
+		case EXTERNALPROGRAM_KEY_COMMAND:
 		{
 			if(commandLine!=value)
 			{
@@ -567,7 +587,7 @@ bool ExternalProgramFilter::setProperty(  unsigned int key,
 			}
 			break;
 		}
-		case KEY_WORKDIR:
+		case EXTERNALPROGRAM_KEY_WORKDIR:
 		{
 			if(workingDir!=value)
 			{
@@ -581,7 +601,7 @@ bool ExternalProgramFilter::setProperty(  unsigned int key,
 			}
 			break;
 		}
-		case KEY_ALWAYSCACHE:
+		case EXTERNALPROGRAM_KEY_ALWAYSCACHE:
 		{
 			string stripped=stripWhite(value);
 
@@ -602,7 +622,7 @@ bool ExternalProgramFilter::setProperty(  unsigned int key,
 			needUpdate=true;
 			break;
 		}
-		case KEY_CLEANUPINPUT:
+		case EXTERNALPROGRAM_KEY_CLEANUPINPUT:
 		{
 			string stripped=stripWhite(value);
 
@@ -783,7 +803,7 @@ bool echoTest()
 	wxString tmpFilename;
 	tmpFilename=wxFileName::CreateTempFileName(wxT(""));
 	s = string(" echo test > ") + stlStr(tmpFilename);
-	TEST(f->setProperty(KEY_COMMAND,s,needUp),"Set prop");
+	TEST(f->setProperty(EXTERNALPROGRAM_KEY_COMMAND,s,needUp),"Set prop");
 
 	//Simulate some data to send to the filter
 	vector<const FilterStreamData*> streamIn,streamOut;
@@ -849,8 +869,8 @@ bool posTest()
 
 	ASSERT(tmpFilename.size());
 	
-	TEST(f->setProperty(KEY_COMMAND,s,needUp),"Set prop");
-	TEST(f->setProperty(KEY_WORKDIR,stlStr(tmpDir),needUp),"Set prop");
+	TEST(f->setProperty(EXTERNALPROGRAM_KEY_COMMAND,s,needUp),"Set prop");
+	TEST(f->setProperty(EXTERNALPROGRAM_KEY_WORKDIR,stlStr(tmpDir),needUp),"Set prop");
 	//Simulate some data to send to the filter
 	vector<const FilterStreamData*> streamIn,streamOut;
 	streamIn.push_back(someData.get());
@@ -885,6 +905,30 @@ bool posTest()
 }
 
 
+bool ExternalProgramFilter::substituteTest()
+{
+	
+	vector<string> ionNames,plotNames;
+	plotNames.push_back("some Plot.xy");
+	ionNames.push_back("my \"pos file.pos");
+
+	string commandLine;
+	commandLine="echo \"My ions are \'%i\'\"";
+
+
+	string resultString;
+	TEST(!substituteVariables(commandLine,ionNames,plotNames,resultString),"substitution fail");
+	TEST(resultString == "echo \"My ions are \'my \"pos file.pos\'\"","substitution fail");
+
+	commandLine=" echo (\"%i\")";
+	TEST(!substituteVariables(commandLine,ionNames,plotNames,resultString),"substitution fail");
+	TEST(resultString == " echo (\"my \"pos file.pos\")","substitution fail");
+
+
+	return true;
+
+}
+
 bool ExternalProgramFilter::runUnitTests() 
 {
 	if(!echoTest())
@@ -893,6 +937,9 @@ bool ExternalProgramFilter::runUnitTests()
 	if(!posTest())
 		return false;
 
+	if(!substituteTest())
+		return false;
+
 	return true;
 }
 
