@@ -24,7 +24,6 @@
 #include "common/stringFuncs.h"
 #include "common/xmlHelper.h"
 
-#include "backend/APT/APTClasses.h"
 #include "backend/APT/APTRanges.h"
 
 //QHull library
@@ -50,6 +49,8 @@ enum
 	HULL_ERR_ENUM_END
 };
 
+const size_t PROGRESS_REDUCE=5000;
+
 //serialise 3D std::vectors to specified output stream in XML format
 void writeVectorsXML(std::ostream &f, const char *containerName,
 		const std::vector<Point3D> &vectorParams, unsigned int depth);
@@ -127,4 +128,8 @@ unsigned int computeConvexHull(const std::vector<const FilterStreamData*> &data,
 unsigned int computeConvexHull(const std::vector<Point3D> &data, 
 			unsigned int *progress, bool (*callback)(bool), 
 			std::vector<Point3D> &hullPts, bool freeHull=true);
+
+//Draw a colour bar
+DrawColourBarOverlay *makeColourBar(float minV, float maxV,size_t nColours,size_t colourMap, bool reverseMap=false) ;
+
 #endif
