@@ -178,13 +178,9 @@ unsigned int IonDownsampleFilter::refresh(const std::vector<const FilterStreamDa
 	//use the cached copy if we have it.
 	if(cacheOK)
 	{
-		for(size_t ui=0;ui<dataIn.size();ui++)
-		{
-			if(dataIn[ui]->getStreamType() != STREAM_TYPE_IONS)
-				getOut.push_back(dataIn[ui]);
-		}
-		for(size_t ui=0;ui<filterOutputs.size();ui++)
-			getOut.push_back(filterOutputs[ui]);
+		propagateStreams(dataIn,getOut,STREAM_TYPE_IONS,true);
+		propagateCache(getOut);
+
 		return 0;
 	}
 
@@ -543,11 +539,10 @@ bool IonDownsampleFilter::setProperty(  unsigned int key,
 		{
 			string stripped=stripWhite(value);
 
-			if(!(stripped == "1"|| stripped == "0"))
-				return false;
-
 			bool lastVal=fixedNumOut;
-			fixedNumOut=(stripped=="1");
+
+			if(!boolStrDec(stripped,fixedNumOut))
+				return false;
 
 			//if the result is different, the
 			//cache should be invalidated
@@ -603,12 +598,9 @@ bool IonDownsampleFilter::setProperty(  unsigned int key,
 		{
 			string stripped=stripWhite(value);
 
-			if(!(stripped == "1"|| stripped == "0"))
-				return false;
-
 			bool lastVal=perSpecies;
-
-			perSpecies=(stripped=="1");
+			if(!boolStrDec(stripped,perSpecies))
+				return false;
 
 			//if the result is different, the
 			//cache should be invalidated
@@ -737,12 +729,8 @@ bool IonDownsampleFilter::readState(xmlNodePtr &nodePtr, const std::string &stat
 	//Retrieve number out (yes/no) mode
 	if(!XMLGetNextElemAttrib(nodePtr,tmpStr,"fixednumout","value"))
 		return false;
-	
-	if(tmpStr == "1") 
-		fixedNumOut=true;
-	else if(tmpStr== "0")
-		fixedNumOut=false;
-	else
+
+	if(!boolStrDec(tmpStr,fixedNumOut))
 		return false;
 	//===
 		
@@ -754,20 +742,22 @@ bool IonDownsampleFilter::readState(xmlNodePtr &nodePtr, const std::string &stat
 	if(fraction < 0.0f || fraction > 1.0f)
 		return false;
 	//===
-	
+
+	//Retreive maxafterfilter
+	//---
+	if(!XMLGetNextElemAttrib(nodePtr,maxAfterFilter,"maxafterfilter","value"))
+		return false;
+	//---
 	
 	//Retrieve "perspecies" attrib
 	if(!XMLGetNextElemAttrib(nodePtr,tmpStr,"perspecies","value"))
 		return false;
-	
-	if(tmpStr == "1") 
-		perSpecies=true;
-	else if(tmpStr== "0")
-		perSpecies=false;
-	else
+
+	if(!boolStrDec(tmpStr,perSpecies))
 		return false;
 
 	//Retrieve the ion per-species fractions
+	//--
 	if(XMLHelpFwdToElem(nodePtr,"fractions"))
 		return false;
 
@@ -782,6 +772,8 @@ bool IonDownsampleFilter::readState(xmlNodePtr &nodePtr, const std::string &stat
 	if(!readScalarsXML(nodePtr,ionLimits))
 		return false;
 
+	//--
+	
 	if(ionLimits.size()!=ionFractions.size())
 		return false;
 
