@@ -59,7 +59,7 @@ bool getRectilinearBounds(const std::vector<const FilterStreamData *> &dataIn, B
 			if(ions->data.size() >1)
 			{
 				ions = (const IonStreamData*)dataIn[ui];
-				c=getIonDataLimits(ions->data);
+				IonHit::getBoundCube(ions->data,c);
 
 				if(c.isValid())
 				{
@@ -176,8 +176,6 @@ Filter *IonInfoFilter::cloneUncached() const
 	p->cache=cache;
 	p->cacheOK=false;
 	p->userString=userString;
-	p->wantVolume=wantVolume;
-	p->volumeAlgorithm=volumeAlgorithm;
 	return p;
 }
 
@@ -233,15 +231,13 @@ unsigned int IonInfoFilter::refresh(const std::vector<const FilterStreamData *>
 			str=std::string(TRANS("--Counts--") );
 			consoleOutput.push_back(str);
 			
-			size_t totalRanged;
+			//sum all ions *except* the unranged.
+			for(size_t ui=0;ui<numIons.size()-1;ui++)
+				numRanged+=numIons[ui];
+				
 			if(wantNormalise)
 			{
-				totalRanged=0;
-				//sum all ions *except* the unranged.
-				for(size_t ui=0;ui<numIons.size()-1;ui++)
-					totalRanged+=numIons[ui];
-				
-				stream_cast(str,totalRanged);
+				stream_cast(str,numRanged);
 				str=TRANS("Total Ranged\t")+str;
 			}
 			else
@@ -257,8 +253,8 @@ unsigned int IonInfoFilter::refresh(const std::vector<const FilterStreamData *>
 			{
 				if(wantNormalise)
 				{
-					if(totalRanged)
-						stream_cast(str,((float)numIons[ui])/(float)totalRanged);
+					if(numRanged)
+						stream_cast(str,((float)numIons[ui])/(float)numRanged);
 					else
 						str=TRANS("n/a");
 				}
@@ -278,9 +274,6 @@ unsigned int IonInfoFilter::refresh(const std::vector<const FilterStreamData *>
 			str=std::string("----------");
 			consoleOutput.push_back(str);
 	
-			
-			for(size_t ui=0;ui<numIons.size();ui++)
-				numRanged++;
 		}
 		else
 		{
@@ -558,7 +551,7 @@ void IonInfoFilter::setPropFromBinding(const SelectionBinding &b)
 }
 
 unsigned int IonInfoFilter::convexHullEstimateVol(const vector<const FilterStreamData*> &data, 
-								float &volume,bool (*callback)(bool))const
+								float &volume,bool (*callback)(bool))
 {
 	volume=0;
 
@@ -832,7 +825,7 @@ bool volumeBoxTest()
 	f->getConsoleStrings(consoleStrings); 
 	
 	//weak test for the console string size
-	TEST(consoleStrings.size(), "console strings existance test");
+	TEST(consoleStrings.size(), "console strings existence test");
 
 
 	//Ensure that the rectilinear volume is the same as
@@ -901,7 +894,7 @@ bool volumeSphereTest()
 	f->getConsoleStrings(consoleStrings); 
 
 	//weak test for the console string size
-	TEST(consoleStrings.size(), "console strings existance test");
+	TEST(consoleStrings.size(), "console strings existence test");
 
 
 	float volMeasure,volReal;
@@ -926,7 +919,7 @@ bool volumeSphereTest()
 	volReal =4.0f/3.0f*M_PI*OUTLINE_RADIUS*OUTLINE_RADIUS*OUTLINE_RADIUS;
 	TEST(fabs(volMeasure -volReal) < 0.05*volReal, "volume test, convex est. of sphere");
 	
-	TEST(consoleStrings.size(), "console strings existance test");
+	TEST(consoleStrings.size(), "console strings existence test");
 
 	delete d;
 	delete f;
