@@ -19,15 +19,10 @@
 
 #include "filterCommon.h"
 
-enum
-{
-	KEY_RANGE_ACTIVE=1,
-	KEY_DROP_UNRANGED,
-	KEY_RANGE_FILENAME,
-	KEY_RANGE_IONID,
-	KEY_ENABLE_ALL_IONS, //Limited to ~100K ions
-	KEY_ENABLE_ALL_RANGES=100000,
-};
+#include <algorithm>
+#include <map>
+
+using std::map;
 
 const unsigned int NUM_ROWS_ION=3;
 const unsigned int NUM_ROWS_RANGE=4;
@@ -71,12 +66,17 @@ Filter *RangeFileFilter::cloneUncached() const
 void RangeFileFilter::initFilter(const std::vector<const FilterStreamData *> &dataIn,
 				std::vector<const FilterStreamData *> &dataOut)
 {
+
 	//Copy any input, except range files to output
 	for(size_t ui=0;ui<dataIn.size();ui++)
 	{
 		if(dataIn[ui]->getStreamType() != STREAM_TYPE_RANGE)
 			dataOut.push_back(dataIn[ui]);
 	}
+	
+	//Reset any changed number of ions
+	ASSERT(rng.getNumRanges() ==  enabledRanges.size() &&
+			rng.getNumIons() == enabledIons.size());
 
 	//Create a rangestream data to push through the init phase
 	if(rng.getNumIons() && rng.getNumRanges())
@@ -102,329 +102,390 @@ unsigned int RangeFileFilter::refresh(const std::vector<const FilterStreamData *
 	//use the cached copy of the data if we have it.
 	if(cacheOK)
 	{
-		for(unsigned int ui=0;ui<filterOutputs.size(); ui++)
-			getOut.push_back(filterOutputs[ui]);
+		propagateCache(getOut);
 
-		for(unsigned int ui=0;ui<dataIn.size() ;ui++)
-		{
-			//We don't cache anything but our modification
-			//to the ion stream data types. so we propagate
-			//these.
-			if(dataIn[ui]->getStreamType() != STREAM_TYPE_IONS)
-				getOut.push_back(dataIn[ui]);
-		}
+		//We don't cache anything but our modification
+		//to the ion stream data types. so we propagate
+		//these.
+		propagateStreams(dataIn,getOut,STREAM_TYPE_IONS,false);
 			
 		return 0;
 	}
 
-	vector<IonStreamData *> d;
 
-	//Split the output up into chunks, one for each range, 
-	//Extra 1 for unranged ions	
-	d.resize(rng.getNumIons()+1);
+	ASSERT(enabledRanges.size() == rng.getNumRanges());
+	ASSERT(enabledIons.size() == rng.getNumIons());
 
-	//Generate output filter streams. 
-	for(unsigned int ui=0;ui<d.size(); ui++)
+	//See if we have enabled ranges and ions
+	bool haveEnabled;
+	haveEnabled= (std::find(enabledRanges.begin(),
+			   	enabledRanges.end(),(char)1) !=enabledRanges.end()) && 
+			(std::find(enabledIons.begin(),
+			   	enabledIons.end(),(char)1) !=enabledIons.end());
+
+	//Nothing enabled? Nothing to do!
+	if(!(!haveEnabled && dropUnranged))
 	{
-		d[ui] = new IonStreamData;
-		d[ui]->parent=this;
-	}
+		vector<IonStreamData *> d;
 
-	bool haveDefIonColour=false;
-	//GCC complains about this, but this is protected by haveDefIonColour.
-	RGBf defIonColour;
-
-	//Try to maintain ion size if possible
-	bool haveIonSize,sameSize; // have we set the ionSize?
-	float ionSize;
-	haveIonSize=false;
-	sameSize=true;
+		//Split the output up into chunks, one for each range, 
+		//Extra 1 for unranged ions	
+		d.resize(rng.getNumIons()+1);
 
 
-	progress.step=1;
-	progress.filterProgress=0;
-	progress.stepName=TRANS("Pre-Allocate");
-	progress.maxStep=2;	
+		bool haveDefIonColour=false;
+		//GCC complains about this, but this is protected by haveDefIonColour.
+		RGBf defIonColour;
 
-	vector<size_t> dSizes;
-	dSizes.resize(d.size(),0);
-	size_t totalSize=numElements(dataIn);
-	
-	//Step 1: Do a first sweep to obtain range sizes needed
-	// then reserve the same amount of mem as we need on the output
-	//========================
-	for(unsigned int ui=0;ui<dataIn.size() ;ui++)
-	{
-		switch(dataIn[ui]->getStreamType())
+		//Try to maintain ion size if possible
+		bool haveIonSize,sameSize; // have we set the ionSize?
+		float ionSize;
+		haveIonSize=false;
+		sameSize=true;
+
+
+		progress.step=1;
+		progress.filterProgress=0;
+		progress.stepName=TRANS("Pre-Allocate");
+		progress.maxStep=2;	
+
+		vector<size_t> dSizes;
+		dSizes.resize(d.size(),0);
+		size_t totalSize=numElements(dataIn);
+
+
+		//Check to see if there are any enabled ranges
+		
+		//Generate output filter streams. 
+		for(unsigned int ui=0;ui<d.size(); ui++)
 		{
-			case STREAM_TYPE_IONS: 
-			{
-
-#ifdef _OPENMP
-				//Create a unique array for each thread, so they don't try
-				//to modify the same data structure
-				unsigned int nT =omp_get_max_threads(); 
-				vector<size_t> *dSizeArr = new vector<size_t>[nT];
-				for(unsigned int uk=0;uk<nT;uk++)
-					dSizeArr[uk].resize(dSizes.size(),0);
-#endif
-				const IonStreamData *src = ((const IonStreamData *)dataIn[ui]);
-				size_t n=0;
-
-
-				unsigned int curProg=NUM_CALLBACK;
-				bool spin=false;
-				#pragma omp parallel for firstprivate(curProg)
-				for(size_t uj=0; uj<src->data.size();uj++)
-				{
-#ifdef _OPENMP
-					unsigned int thisT=omp_get_thread_num();
-#endif
-					if(spin)
-						continue;
-					
-					//get the range ID for this particular ion.
-					unsigned int rangeID;
-					rangeID=rng.getRangeID(src->data[uj].getMassToCharge());
-
-					//If ion is unranged, then it will have a rangeID of -1
-					if(rangeID != (unsigned int)-1 && enabledRanges[rangeID] )
-					{
-						unsigned int ionID=rng.getIonID(rangeID);
-
-						//if we are going to keep the ion
-						//then increment this array size
-						if(enabledIons[ionID])
-						{
-							#ifdef _OPENMP
-								dSizeArr[thisT][ionID]++;
-							#else
-								dSizes[ionID]++;
-							#endif
-
-						}
-					}
-					
-					//update progress periodically
-					if(!curProg--)
-					{
-#pragma omp critical
-						{
-						n+=NUM_CALLBACK;
-						progress.filterProgress= (unsigned int)((float)(n)/((float)totalSize)*100.0f);
-						curProg=NUM_CALLBACK;
-
-
-						if(!(*callback)(false))
-							spin=true;
-						}
-					}
-				}
-
-				if(spin)
-					return RANGEFILE_ABORT_FAIL;
-#ifdef _OPENMP
-				//Merge the arrays back together
-				for(unsigned int uk=0;uk<nT;uk++)
-				{
-					for(unsigned int uj=0;uj<dSizes.size();uj++)
-						dSizes[uj] = dSizes[uj]+dSizeArr[uk][uj];
-				}
-				delete[] dSizeArr;
-#endif
-
-			}
+			d[ui] = new IonStreamData;
+			d[ui]->parent=this;
 		}
-	}
-
-
-	//reserve the vector to the exact size we need
-	try
-	{
-		for(size_t ui=0;ui<d.size();ui++)
-			d[ui]->data.reserve(dSizes[ui]);
-	}
-	catch(std::bad_alloc)
-	{
-		for(size_t ui=0;ui<d.size();ui++)
-			delete d[ui];
-		return RANGEFILE_BAD_ALLOC;
-	}
-
-	dSizes.clear();
-	//===================================
-
-	//Update progress info
-	progress.step=2;
-	progress.filterProgress=0;
-	progress.stepName=TRANS("Range");
-	size_t n=0;
-
-
-	
-
-	//Step 2: Go through each data stream, if it is an ion stream, range it.
-	//	I tried parallelising this a few different ways, but the linear performance wa simply better.
-	//		- Tried an array of openmp locks
-	//		- Tried keeping a unique offset number and fixing the size of the output vectors
-	//		- Tried straight criticalling the push_back
-	//	 Trying to merge vectors in // is tricky. result was 9-10x slower than linear
-	//=========================================
-	for(unsigned int ui=0;ui<dataIn.size() ;ui++)
-	{
-		switch(dataIn[ui]->getStreamType())
+		
+		if(!haveEnabled)
 		{
-			case STREAM_TYPE_IONS: 
-			{
-				//Set the default (unranged) ion colour, by using
-				//the first input ion colour.
-				if(!haveDefIonColour)
-				{
-					defIonColour.red =  ((IonStreamData *)dataIn[ui])->r;
-					defIonColour.green =  ((IonStreamData *)dataIn[ui])->g;
-					defIonColour.blue =  ((IonStreamData *)dataIn[ui])->b;
-					haveDefIonColour=true;
-				}
-			
-				//Check for ion size consistency	
-				if(haveIonSize)
-				{
-					sameSize &= (fabs(ionSize-((const IonStreamData *)dataIn[ui])->ionSize) 
-									< std::numeric_limits<float>::epsilon());
-				}
-				else
-				{
-					ionSize=((const IonStreamData *)dataIn[ui])->ionSize;
-					haveIonSize=true;
-				}
-
-				unsigned int curProg=NUM_CALLBACK;
-				const size_t off=d.size()-1;
-
-				for(vector<IonHit>::const_iterator it=((const IonStreamData *)dataIn[ui])->data.begin();
-					       it!=((const IonStreamData *)dataIn[ui])->data.end(); ++it)
-				{
-					unsigned int rangeID;
-					rangeID=rng.getRangeID(it->getMassToCharge());
-
-					//If ion is unranged, then it will have a rangeID of -1
-					if(rangeID != (unsigned int)-1)
-					{
-						unsigned int ionID;
-						ionID=rng.getIonID(rangeID);
-
-						//Only retain the ion if the ionID and rangeID are enabled
-						if(enabledRanges[rangeID] && enabledIons[ionID])
-						{
-							ASSERT(ionID < enabledRanges.size());
-
-							d[ionID]->data.push_back(*it);
-						}
-					}
-					else if(!dropUnranged)//If it is unranged, then the rangeID is still -1 (as above).
-					{
-						d[off]->data.push_back(*it);
-					}
-
-					//update progress periodically
-					if(!curProg--)
-					{
-						n+=NUM_CALLBACK;
-						progress.filterProgress= (unsigned int)((float)(n)/((float)totalSize)*100.0f);
-						curProg=NUM_CALLBACK;
-
-						
-						if(!(*callback)(false))
-						{
-							//Free space allocated for output ion streams...
-							for(unsigned int ui=0;ui<d.size();ui++)
-								delete d[ui];
-							return RANGEFILE_ABORT_FAIL;
-						}
-					}
-
-				}
-
-
-			}
-				break;
-			case STREAM_TYPE_RANGE:
-				//Purposely do nothing. This blocks propagation of other ranges
-				//i.e. there can only be one in any given node of the tree.
-				break;
-			default:
-				getOut.push_back(dataIn[ui]);
-				break;
-		}
-	}
-	//=========================================
-
-
-	//Step 3 : Set up any properties for the output streams that we need, like colour, size, caching. Trim any empty results.
-	//======================================
-	//Set the colour of the output ranges
-	//and whether to cache.
-	for(unsigned int ui=0; ui<rng.getNumIons(); ui++)
-	{
-		RGBf rngCol;
-		rngCol = rng.getColour(ui);
-		d[ui]->r=rngCol.red;
-		d[ui]->g=rngCol.green;
-		d[ui]->b=rngCol.blue;
-		d[ui]->a=1.0;
-
-	}
-
-	//If all the ions are the same size, then propagate
-	//Otherwise use the default ionsize
-	if(haveIonSize && sameSize)
-	{
-		for(unsigned int ui=0;ui<d.size();ui++)
-			d[ui]->ionSize=ionSize;
-	}
-	
-	//Set the unranged colour
-	if(haveDefIonColour && d.size())
-	{
-		d[d.size()-1]->r = defIonColour.red;
-		d[d.size()-1]->g = defIonColour.green;
-		d[d.size()-1]->b = defIonColour.blue;
-		d[d.size()-1]->a = 1.0f;
-	}
-
-	//remove any zero sized ranges
-	for(unsigned int ui=0;ui<d.size();)
-	{
-		if(!(d[ui]->data.size()))
-		{
-			delete d[ui];
-			std::swap(d[ui],d.back());
-			d.pop_back();
+			//There are no enabled ranges at all.
+			dSizes.back()=totalSize;
 		}
 		else
-			ui++;
-	}
-
-	//======================================
-
-	//Having ranged all streams, merge them back into one ranged stream.
-	if(cache)
-	{
-		for(unsigned int ui=0;ui<d.size(); ui++)
 		{
-			d[ui]->cached=1; //IMPORTANT: ->cached must be set PRIOR to push back
-			filterOutputs.push_back(d[ui]);
+			//Step 1: Do a first sweep to obtain range sizes needed
+			// then reserve the same amount of mem as we need on the output
+			//========================
+			for(unsigned int ui=0;ui<dataIn.size() ;ui++)
+			{
+				switch(dataIn[ui]->getStreamType())
+				{
+					case STREAM_TYPE_IONS: 
+					{
+
+#ifdef _OPENMP
+						//Create a unique array for each thread, so they don't try
+						//to modify the same data structure
+						unsigned int nT =omp_get_max_threads(); 
+						vector<size_t> *dSizeArr = new vector<size_t>[nT];
+						for(unsigned int uk=0;uk<nT;uk++)
+							dSizeArr[uk].resize(dSizes.size(),0);
+#endif
+						const IonStreamData *src = ((const IonStreamData *)dataIn[ui]);
+						size_t n=0;
+
+
+						unsigned int curProg=NUM_CALLBACK;
+						bool spin=false;
+						#pragma omp parallel for firstprivate(curProg)
+						for(size_t uj=0; uj<src->data.size();uj++)
+						{
+#ifdef _OPENMP
+							unsigned int thisT=omp_get_thread_num();
+#endif
+							if(spin)
+								continue;
+							
+							//get the range ID for this particular ion.
+							unsigned int rangeID;
+							rangeID=rng.getRangeID(src->data[uj].getMassToCharge());
+
+							//If ion is unranged, then it will have a rangeID of -1
+							if(rangeID != (unsigned int)-1 && enabledRanges[rangeID] )
+							{
+								unsigned int ionID=rng.getIonID(rangeID);
+
+								//if we are going to keep the ion
+								//then increment this array size
+								if(enabledIons[ionID])
+								{
+									#ifdef _OPENMP
+										dSizeArr[thisT][ionID]++;
+									#else
+										dSizes[ionID]++;
+									#endif
+
+								}
+							}
+							
+							//update progress periodically
+							if(!curProg--)
+							{
+#pragma omp critical
+								{
+								n+=NUM_CALLBACK;
+								progress.filterProgress= (unsigned int)((float)(n)/((float)totalSize)*100.0f);
+								curProg=NUM_CALLBACK;
+
+
+								if(!(*callback)(false))
+									spin=true;
+								}
+							}
+						}
+
+						if(spin)
+							return RANGEFILE_ABORT_FAIL;
+#ifdef _OPENMP
+						//Merge the arrays back together
+						for(unsigned int uk=0;uk<nT;uk++)
+						{
+							for(unsigned int uj=0;uj<dSizes.size();uj++)
+								dSizes[uj] = dSizes[uj]+dSizeArr[uk][uj];
+						}
+						delete[] dSizeArr;
+#endif
+
+					}
+				}
+			}
 		}
-	}
-	else
-	{
+
+		//reserve the vector to the exact size we need
+		try
+		{
+			for(size_t ui=0;ui<d.size();ui++)
+				d[ui]->data.reserve(dSizes[ui]);
+		}
+		catch(std::bad_alloc)
+		{
+			for(size_t ui=0;ui<d.size();ui++)
+				delete d[ui];
+			return RANGEFILE_BAD_ALLOC;
+		}
+
+		dSizes.clear();
+		//===================================
+
+		//Update progress info
+		progress.step=2;
+		progress.filterProgress=0;
+		progress.stepName=TRANS("Range");
+
+
+		
+
+		//Step 2: Go through each data stream, if it is an ion stream, range it.
+		//=========================================
+		if(haveEnabled)
+		{
+			//	I tried parallelising this a few different ways, but the linear performance wa simply better.
+			//		- Tried an array of openmp locks
+			//		- Tried keeping a unique offset number and fixing the size of the output vectors
+			//		- Tried straight criticalling the push_back
+			//	 Trying to merge vectors in // is tricky. result was 9-10x slower than linear
+		
+			size_t n=0;
+			for(unsigned int ui=0;ui<dataIn.size() ;ui++)
+			{
+				switch(dataIn[ui]->getStreamType())
+				{
+					case STREAM_TYPE_IONS: 
+					{
+						//Set the default (unranged) ion colour, by using
+						//the first input ion colour.
+						if(!haveDefIonColour)
+						{
+							defIonColour.red =  ((IonStreamData *)dataIn[ui])->r;
+							defIonColour.green =  ((IonStreamData *)dataIn[ui])->g;
+							defIonColour.blue =  ((IonStreamData *)dataIn[ui])->b;
+							haveDefIonColour=true;
+						}
+					
+						//Check for ion size consistency	
+						if(haveIonSize)
+						{
+							sameSize &= (fabs(ionSize-((const IonStreamData *)dataIn[ui])->ionSize) 
+											< std::numeric_limits<float>::epsilon());
+						}
+						else
+						{
+							ionSize=((const IonStreamData *)dataIn[ui])->ionSize;
+							haveIonSize=true;
+						}
+
+						unsigned int curProg=NUM_CALLBACK;
+						const size_t off=d.size()-1;
+
+						for(vector<IonHit>::const_iterator it=((const IonStreamData *)dataIn[ui])->data.begin();
+							       it!=((const IonStreamData *)dataIn[ui])->data.end(); ++it)
+						{
+							unsigned int rangeID;
+							rangeID=rng.getRangeID(it->getMassToCharge());
+
+							//If ion is unranged, then it will have a rangeID of -1
+							if(rangeID != (unsigned int)-1)
+							{
+								unsigned int ionID;
+								ionID=rng.getIonID(rangeID);
+
+								//Only retain the ion if the ionID and rangeID are enabled
+								if(enabledRanges[rangeID] && enabledIons[ionID])
+								{
+									ASSERT(ionID < enabledRanges.size());
+
+									d[ionID]->data.push_back(*it);
+								}
+							}
+							else if(!dropUnranged)//If it is unranged, then the rangeID is still -1 (as above).
+							{
+								d[off]->data.push_back(*it);
+							}
+
+							//update progress periodically
+							if(!curProg--)
+							{
+								n+=NUM_CALLBACK;
+								progress.filterProgress= (unsigned int)((float)(n)/((float)totalSize)*100.0f);
+								curProg=NUM_CALLBACK;
+
+								
+								if(!(*callback)(false))
+								{
+									//Free space allocated for output ion streams...
+									for(unsigned int ui=0;ui<d.size();ui++)
+										delete d[ui];
+									return RANGEFILE_ABORT_FAIL;
+								}
+							}
+
+						}
+
+
+					}
+						break;
+					case STREAM_TYPE_RANGE:
+						//Purposely do nothing. This blocks propagation of other ranges
+						//i.e. there can only be one in any given node of the tree.
+						break;
+					default:
+						getOut.push_back(dataIn[ui]);
+						break;
+				}
+			}
+		}
+		else
+		{
+			//We have no ranges enabled, thus everything must go in the "unranged" section
+			d.back()->data.resize(totalSize);
+
+			size_t off=0;
+			for(unsigned int ui=0;ui<dataIn.size() ;ui++)
+			{
+				switch(dataIn[ui]->getStreamType())
+				{
+					case STREAM_TYPE_IONS:
+					{
+						const vector<IonHit> &ionHitVec=((const IonStreamData *)dataIn[ui])->data;
+						vector<IonHit> &outputVec=(d.back())->data;
+#pragma omp parallel for
+						for(size_t ui=0;ui<ionHitVec.size();ui++)
+							outputVec[off+ui]=ionHitVec[ui];
+						break;
+					}
+					case STREAM_TYPE_RANGE:
+						break;
+					default:
+						getOut.push_back(dataIn[ui]);
+						break;
+				}
+			}
+
+			if(!(*callback)(false))
+			{
+				//Free space allocated for output ion streams...
+				for(unsigned int ui=0;ui<d.size();ui++)
+					delete d[ui];
+				return RANGEFILE_ABORT_FAIL;
+			}
+		}
+		//=========================================
+
+
+		//Step 3 : Set up any properties for the output streams that we need, like colour, size, caching. Trim any empty results.
+		//======================================
+		//Set the colour of the output ranges
+		//and whether to cache.
+		for(unsigned int ui=0; ui<rng.getNumIons(); ui++)
+		{
+			RGBf rngCol;
+			rngCol = rng.getColour(ui);
+			d[ui]->r=rngCol.red;
+			d[ui]->g=rngCol.green;
+			d[ui]->b=rngCol.blue;
+			d[ui]->a=1.0;
+
+		}
+
+		//If all the ions are the same size, then propagate
+		//Otherwise use the default ionsize
+		if(haveIonSize && sameSize)
+		{
+			for(unsigned int ui=0;ui<d.size();ui++)
+				d[ui]->ionSize=ionSize;
+		}
+		
+		//Set the unranged colour
+		if(haveDefIonColour && d.size())
+		{
+			d[d.size()-1]->r = defIonColour.red;
+			d[d.size()-1]->g = defIonColour.green;
+			d[d.size()-1]->b = defIonColour.blue;
+			d[d.size()-1]->a = 1.0f;
+		}
+
+		//remove any zero sized ranges
+		for(unsigned int ui=0;ui<d.size();)
+		{
+			if(!(d[ui]->data.size()))
+			{
+				delete d[ui];
+				std::swap(d[ui],d.back());
+				d.pop_back();
+			}
+			else
+				ui++;
+		}
+
+		//======================================
+
+		//Having ranged all streams, merge them back into one ranged stream.
+		if(cache)
+		{
+			for(unsigned int ui=0;ui<d.size(); ui++)
+			{
+				d[ui]->cached=1; //IMPORTANT: ->cached must be set PRIOR to push back
+				filterOutputs.push_back(d[ui]);
+			}
+		}
+		else
+		{
+			for(unsigned int ui=0;ui<d.size(); ui++)
+				d[ui]->cached=0; //IMPORTANT: ->cached must be set PRIOR to push back
+			cacheOK=false;
+		}
+		
 		for(unsigned int ui=0;ui<d.size(); ui++)
-			d[ui]->cached=0; //IMPORTANT: ->cached must be set PRIOR to push back
-		cacheOK=false;
+			getOut.push_back(d[ui]);
 	}
-	
-	for(unsigned int ui=0;ui<d.size(); ui++)
-		getOut.push_back(d[ui]);
+
 
 	//Put out rangeData
 	RangeStreamData *rngData=new RangeStreamData;
@@ -481,6 +542,8 @@ void RangeFileFilter::setRangeData(const RangeFile &rngNew)
 	//Turn all ions to "on" 
 	for(unsigned int ui=0;ui<enabledIons.size(); ui++)
 		enabledIons[ui]=(char)1;
+
+	clearCache();
 }
 
 size_t RangeFileFilter::numBytesForCache(size_t nObjects) const
@@ -504,7 +567,7 @@ void RangeFileFilter::getProperties(FilterPropGroup &p) const
 	prop.name=TRANS("File");
 	prop.type=PROPERTY_TYPE_STRING;
 	prop.helpText=TRANS("File to use for range data");
-	prop.key=KEY_RANGE_FILENAME;
+	prop.key=RANGE_KEY_RANGE_FILENAME;
 	prop.data=rngName;
 
 	p.addProperty(prop,curGroup);	
@@ -518,7 +581,7 @@ void RangeFileFilter::getProperties(FilterPropGroup &p) const
 	prop.name=TRANS("Drop unranged");
 	prop.type=PROPERTY_TYPE_BOOL;
 	prop.helpText=TRANS("Remove unranged points when generating output");
-	prop.key=KEY_DROP_UNRANGED;
+	prop.key=RANGE_KEY_DROP_UNRANGED;
 	prop.data=tmpStr;
 
 	p.addProperty(prop,curGroup);
@@ -541,7 +604,7 @@ void RangeFileFilter::getProperties(FilterPropGroup &p) const
 		prop.helpText=TRANS("Enable/disable all ions at once");
 		prop.data=str;
 		prop.type=PROPERTY_TYPE_BOOL;
-		prop.key=KEY_ENABLE_ALL_IONS;
+		prop.key=RANGE_KEY_ENABLE_ALL_IONS;
 
 		p.addProperty(prop,curGroup);
 
@@ -557,7 +620,7 @@ void RangeFileFilter::getProperties(FilterPropGroup &p) const
 			prop.helpText=TRANS("Enable/disable specified ion");
 			prop.data=rng.getName(ui);
 			prop.type=PROPERTY_TYPE_STRING;
-			prop.key=NUM_ROWS_ION*ui+1+KEY_ENABLE_ALL_IONS;
+			prop.key=NUM_ROWS_ION*ui+1+RANGE_KEY_ENABLE_ALL_IONS;
 
 			p.addProperty(prop,curGroup);
 
@@ -571,7 +634,7 @@ void RangeFileFilter::getProperties(FilterPropGroup &p) const
 			prop.type=PROPERTY_TYPE_BOOL;
 			prop.helpText=TRANS("If true, ion is used in output");
 			prop.data=str;
-			prop.key=NUM_ROWS_ION*ui+2+KEY_ENABLE_ALL_IONS;
+			prop.key=NUM_ROWS_ION*ui+2+RANGE_KEY_ENABLE_ALL_IONS;
 		
 			p.addProperty(prop,curGroup);
 
@@ -587,7 +650,7 @@ void RangeFileFilter::getProperties(FilterPropGroup &p) const
 			prop.data=thisCol;
 			prop.type=PROPERTY_TYPE_COLOUR;
 			prop.helpText=TRANS("Colour used to represent ion");
-			prop.key=NUM_ROWS_ION*ui+3+KEY_ENABLE_ALL_IONS;
+			prop.key=NUM_ROWS_ION*ui+3+RANGE_KEY_ENABLE_ALL_IONS;
 
 			p.addProperty(prop,curGroup);
 		}
@@ -614,7 +677,7 @@ void RangeFileFilter::getProperties(FilterPropGroup &p) const
 		prop.helpText=TRANS("Enable/disable all ranges");
 		prop.data=str;
 		prop.type=PROPERTY_TYPE_BOOL;
-		prop.key=KEY_ENABLE_ALL_RANGES;
+		prop.key=RANGE_KEY_ENABLE_ALL_RANGES;
 	
 		p.addProperty(prop,curGroup);
 		
@@ -633,14 +696,14 @@ void RangeFileFilter::getProperties(FilterPropGroup &p) const
 			prop.data=str;
 			prop.type=PROPERTY_TYPE_BOOL;
 			prop.helpText=TRANS("Enable/disable specified range (ion must also be enabled to activiate range)");
-			prop.key=KEY_ENABLE_ALL_RANGES+NUM_ROWS_RANGE*ui+1;
+			prop.key=RANGE_KEY_ENABLE_ALL_RANGES+NUM_ROWS_RANGE*ui+1;
 			p.addProperty(prop,curGroup);
 
 			prop.name=string(TRANS("Ion ")) + suffix;
 			prop.data=rng.getName(rng.getIonID(ui));
 			prop.type=PROPERTY_TYPE_STRING;
 			prop.helpText=TRANS("Name of ion associate to this range");
-			prop.key=KEY_ENABLE_ALL_RANGES+NUM_ROWS_RANGE*ui+2;
+			prop.key=RANGE_KEY_ENABLE_ALL_RANGES+NUM_ROWS_RANGE*ui+2;
 			p.addProperty(prop,curGroup);
 
 			std::pair<float,float > thisRange;
@@ -652,7 +715,7 @@ void RangeFileFilter::getProperties(FilterPropGroup &p) const
 			prop.data=rangeVal;
 			prop.type=PROPERTY_TYPE_REAL;
 			prop.helpText=TRANS("Start value for range");
-			prop.key=KEY_ENABLE_ALL_RANGES + NUM_ROWS_RANGE*ui +3;
+			prop.key=RANGE_KEY_ENABLE_ALL_RANGES + NUM_ROWS_RANGE*ui +3;
 			p.addProperty(prop,curGroup);
 		
 			stream_cast(rangeVal,thisRange.second);
@@ -660,7 +723,7 @@ void RangeFileFilter::getProperties(FilterPropGroup &p) const
 			prop.data=rangeVal;
 			prop.type=PROPERTY_TYPE_REAL;
 			prop.helpText=TRANS("Stopping value for range`");
-			prop.key=KEY_ENABLE_ALL_RANGES+NUM_ROWS_RANGE*ui+4;
+			prop.key=RANGE_KEY_ENABLE_ALL_RANGES+NUM_ROWS_RANGE*ui+4;
 			p.addProperty(prop,curGroup);
 		}
 		p.setGroupTitle(curGroup,TRANS("Ranges"));
@@ -678,7 +741,7 @@ bool RangeFileFilter::setProperty(unsigned int key,
 
 	switch(key)
 	{
-		case KEY_RANGE_FILENAME:
+		case RANGE_KEY_RANGE_FILENAME:
 		{
 			if(value != rngName)
 			{
@@ -708,7 +771,7 @@ bool RangeFileFilter::setProperty(unsigned int key,
 
 			break;
 		}
-		case KEY_DROP_UNRANGED: //Enable/disable unranged dropping
+		case RANGE_KEY_DROP_UNRANGED: //Enable/disable unranged dropping
 		{
 			unsigned int valueInt;
 			if(stream_cast(valueInt,value))
@@ -716,7 +779,7 @@ bool RangeFileFilter::setProperty(unsigned int key,
 
 			if(valueInt ==0 || valueInt == 1)
 			{
-				if((int)dropUnranged!= valueInt)
+				if(dropUnranged!= (bool)valueInt)
 				{
 					needUpdate=true;
 					dropUnranged=valueInt;
@@ -732,7 +795,7 @@ bool RangeFileFilter::setProperty(unsigned int key,
 
 			break;
 		}	
-		case KEY_ENABLE_ALL_RANGES:
+		case RANGE_KEY_ENABLE_ALL_RANGES:
 		{
 
 			bool allEnable;
@@ -759,7 +822,7 @@ bool RangeFileFilter::setProperty(unsigned int key,
 
 			break;
 		}
-		case KEY_ENABLE_ALL_IONS:
+		case RANGE_KEY_ENABLE_ALL_IONS:
 		{
 
 			bool allEnable;
@@ -787,15 +850,15 @@ bool RangeFileFilter::setProperty(unsigned int key,
 		}
 		default:
 		{
-			if(key < KEY_ENABLE_ALL_RANGES)
+			if(key < RANGE_KEY_ENABLE_ALL_RANGES)
 			{
 				//structured group, each with NUM_ROWS_ION per grouping.
 				//The ion ID is simply the row number/NUM_ROWS_ION.
 				//similarly the element is given by remainder 
-				unsigned int ionID=((key-1)-KEY_ENABLE_ALL_IONS)/NUM_ROWS_ION;
+				unsigned int ionID=((key-1)-RANGE_KEY_ENABLE_ALL_IONS)/NUM_ROWS_ION;
 				ASSERT(ionID < rng.getNumIons());
-				ASSERT(key <=NUM_ROWS_ION*rng.getNumIons()+KEY_ENABLE_ALL_IONS);
-				unsigned int prop = ((key-1)-KEY_ENABLE_ALL_IONS)-ionID*NUM_ROWS_ION;
+				ASSERT(key <=NUM_ROWS_ION*rng.getNumIons()+RANGE_KEY_ENABLE_ALL_IONS);
+				unsigned int prop = ((key-1)-RANGE_KEY_ENABLE_ALL_IONS)-ionID*NUM_ROWS_ION;
 
 				switch(prop)
 				{
@@ -860,8 +923,8 @@ bool RangeFileFilter::setProperty(unsigned int key,
 			}
 			else
 			{
-				unsigned int rangeId=((key-1)-KEY_ENABLE_ALL_RANGES)/NUM_ROWS_RANGE;
-				unsigned int prop = ((key-1)-KEY_ENABLE_ALL_RANGES)-rangeId*NUM_ROWS_RANGE;
+				unsigned int rangeId=((key-1)-RANGE_KEY_ENABLE_ALL_RANGES)/NUM_ROWS_RANGE;
+				unsigned int prop = ((key-1)-RANGE_KEY_ENABLE_ALL_RANGES)-rangeId*NUM_ROWS_RANGE;
 				switch(prop)
 				{
 					//Range active
@@ -1097,7 +1160,8 @@ bool RangeFileFilter::readState(xmlNodePtr &nodePtr, const std::string &stateFil
 	unsigned int ionID;
 	bool enabled;
 	//By default, turn ions off, but use state file to turn them on
-	enabledIons.resize(rng.getNumIons(),false);
+	map<unsigned int ,char> tmpEnabledIons;
+	map<unsigned int, RGBf> tmpCol;
 	while(!XMLHelpFwdToElem(nodePtr,"ion"))
 	{
 		//Get ID value
@@ -1126,7 +1190,7 @@ bool RangeFileFilter::readState(xmlNodePtr &nodePtr, const std::string &stateFil
 		else
 			return false;
 
-		enabledIons[ionID]=enabled;
+		tmpEnabledIons[ionID]=enabled;
 		xmlFree(xmlString);
 		
 		
@@ -1145,10 +1209,11 @@ bool RangeFileFilter::readState(xmlNodePtr &nodePtr, const std::string &stateFil
 		col.red=(float)r/255.0f;
 		col.green=(float)g/255.0f;
 		col.blue=(float)b/255.0f;
-		rng.setColour(ionID,col);	
+		tmpCol[ionID]=col;	
 		xmlFree(xmlString);
 	}
 
+
 	//===
 
 
@@ -1162,7 +1227,7 @@ bool RangeFileFilter::readState(xmlNodePtr &nodePtr, const std::string &stateFil
 	nodePtr=nodePtr->xmlChildrenNode;
 
 	//By default, turn ranges off (cause there are lots of them), and use state to turn them on
-	enabledRanges.resize(rng.getNumRanges(),true);
+	map<unsigned int, char> tmpEnabledRanges;
 	unsigned int rngID;
 	while(!XMLHelpFwdToElem(nodePtr,"range"))
 	{
@@ -1193,10 +1258,24 @@ bool RangeFileFilter::readState(xmlNodePtr &nodePtr, const std::string &stateFil
 			return false;
 
 		xmlFree(xmlString);
-		enabledRanges[rngID]=enabled;
+		tmpEnabledRanges[rngID]=enabled;
 	}
 	//===
 
+	enabledIons.resize(rng.getNumIons(),1);
+	enabledRanges.resize(rng.getNumRanges(),1);
+	
+	//FIXME: HACK: store more data, to confirm validation
+	//If we have the same number of ions, and the same number of ranges,
+	// then update the values
+	if(tmpEnabledIons.size() == rng.getNumIons())
+	{
+		for(size_t ui=0;ui<tmpEnabledIons.size(); ui++)
+			enabledIons[ui] = tmpEnabledIons[ui];
+		
+		for(size_t ui=0;ui<tmpEnabledRanges.size(); ui++)
+			enabledRanges[ui] = tmpEnabledRanges[ui];
+	}
 	return true;
 }
 
