@@ -138,11 +138,10 @@ Filter *TransformFilter::cloneUncached() const
 
 size_t TransformFilter::numBytesForCache(size_t nObjects) const
 {
-	//Say we don't know, we are not going to cache anyway.
-	return (size_t)-1;
+	return nObjects*sizeof(IonHit);
 }
 
-DrawStreamData* TransformFilter::makeMarkerSphere(SelectionDevice<Filter>* &s) const
+DrawStreamData* TransformFilter::makeMarkerSphere(SelectionDevice* &s) const
 {
 	//construct a new primitive, do not cache
 	DrawStreamData *drawData=new DrawStreamData;
@@ -170,10 +169,10 @@ DrawStreamData* TransformFilter::makeMarkerSphere(SelectionDevice<Filter>* &s) c
 	{
 		dS->canSelect=true;
 
-		s=new SelectionDevice<Filter>(this);
+		s=new SelectionDevice(this);
 		SelectionBinding b;
 
-		b.setBinding(SELECT_BUTTON_LEFT,FLAG_CMD,DRAW_SPHERE_BIND_ORIGIN,
+		b.setBinding(SELECT_BUTTON_LEFT,0,DRAW_SPHERE_BIND_ORIGIN,
 		             BINDING_SPHERE_ORIGIN,dS->getOrigin(),dS);
 		b.setInteractionMode(BIND_MODE_POINT3D_TRANSLATE);
 		s->addBinding(b);
@@ -187,37 +186,11 @@ DrawStreamData* TransformFilter::makeMarkerSphere(SelectionDevice<Filter>* &s) c
 unsigned int TransformFilter::refresh(const std::vector<const FilterStreamData *> &dataIn,
 	std::vector<const FilterStreamData *> &getOut, ProgressData &progress, bool (*callback)(bool))
 {
-	//Clear selection devices FIXME: Is this a memory leak???
-	clearDevices();
 	//use the cached copy if we have it.
 	if(cacheOK)
 	{
-		ASSERT(filterOutputs.size());
-		for(unsigned int ui=0;ui<dataIn.size();ui++)
-		{
-			if(dataIn[ui]->getStreamType() != STREAM_TYPE_IONS)
-				getOut.push_back(dataIn[ui]);
-		}
-
-		for(unsigned int ui=0;ui<filterOutputs.size();ui++)
-			getOut.push_back(filterOutputs[ui]);
-		
-		
-		if(filterOutputs.size() && showPrimitive)
-		{
-			//If the user is using a transform mode that requires origin selection 
-			if(showOrigin && (transformMode == MODE_ROTATE ||
-					transformMode == MODE_SCALE_ANISOTROPIC ||
-					transformMode == MODE_SCALE_ISOTROPIC) )
-			{
-				SelectionDevice<Filter> *s;
-				getOut.push_back(makeMarkerSphere(s));
-				if(s)
-					devices.push_back(s);
-			}
-			
-		}
-
+		propagateStreams(dataIn,getOut, STREAM_TYPE_IONS,false);
+		propagateCache(getOut);
 		return 0;
 	}
 
@@ -241,7 +214,7 @@ unsigned int TransformFilter::refresh(const std::vector<const FilterStreamData *
 					ions = (const IonStreamData*)dataIn[ui];
 					if(ions->data.size())
 					{
-						thisB = getIonDataLimits(ions->data);
+						IonHit::getBoundCube(ions->data,thisB);
 						#pragma omp critical
 						masterB.expand(thisB);
 					}
@@ -295,16 +268,40 @@ unsigned int TransformFilter::refresh(const std::vector<const FilterStreamData *
 			transformMode == MODE_SCALE_ANISOTROPIC || 
 			transformMode == MODE_SCALE_ISOTROPIC) )
 	{
-		SelectionDevice<Filter> *s;
-		getOut.push_back(makeMarkerSphere(s));
+		SelectionDevice *s;
+		DrawStreamData *d=makeMarkerSphere(s);
 		if(s)
 			devices.push_back(s);
+		if(cache)
+		{
+			d->cached=1;
+			filterOutputs.push_back(d);
+		}
+		else
+			d->cached=0;
+		
+		getOut.push_back(d);
 	}
 			
 	//Apply the transformations to the incoming 
 	//ion streams, generating new outgoing ion streams with
 	//the modified positions
 	size_t totalSize=numElements(dataIn);
+
+	//If there are no ions, nothing to do.
+	// just copy non-ion input to output 
+	if(!totalSize)
+	{
+		for(unsigned int ui=0;ui<dataIn.size();ui++)
+		{
+			if(dataIn[ui]->getStreamType() == STREAM_TYPE_IONS)
+				continue;
+
+			getOut.push_back(dataIn[ui]);
+		}
+		return 0;
+	}
+
 	if( transformMode != MODE_VALUE_SHUFFLE)
 	{
 		//Dont cross the streams. Why? It would be bad.
@@ -1395,7 +1392,7 @@ bool TransformFilter::setProperty(  unsigned int key,
 				if(value == TRANS(TRANSFORM_MODE_STRING[ui]))
 					tmp=ui;
 			}
-			if(tmp==-1)
+			if(tmp==(size_t)-1)
 			{
 				ASSERT(false); // This should not happen
 				return false;
@@ -1655,7 +1652,7 @@ bool TransformFilter::readState(xmlNodePtr &nodePtr, const std::string &stateFil
 		return false;
 	//====
 	
-	//Retreive vector parameters
+	//Retrieve vector parameters
 	//===
 	if(XMLHelpFwdToElem(nodePtr,"vectorparams"))
 		return false;
@@ -1666,7 +1663,7 @@ bool TransformFilter::readState(xmlNodePtr &nodePtr, const std::string &stateFil
 	//===	
 
 	nodePtr=tmpNode;
-	//Retreive scalar parameters
+	//Retrieve scalar parameters
 	//===
 	if(XMLHelpFwdToElem(nodePtr,"scalarparams"))
 		return false;
@@ -1908,8 +1905,8 @@ bool rotateTest()
 	// however we don't quite have  a sphere, so we could have (at the most extreme,
 	// a cube)
 	BoundCube bc[2];
-	bc[0]=getIonDataLimits(d->data);
-	bc[1]=getIonDataLimits(outData->data);
+	IonHit::getBoundCube(d->data,bc[0]);
+	IonHit::getBoundCube(outData->data,bc[1]);
 
 	float volumeRat;
 	volumeRat = bc[0].volume()/bc[1].volume();
@@ -1974,8 +1971,8 @@ bool translateTest()
 
 	//Bound cube should move exactly as per the translation
 	BoundCube bc[2];
-	bc[0]=getIonDataLimits(d->data);
-	bc[1]=getIonDataLimits(outData->data);
+	IonHit::getBoundCube(d->data,bc[0]);
+	IonHit::getBoundCube(outData->data,bc[1]);
 
 	for(unsigned int ui=0;ui<3;ui++)
 	{
@@ -2041,7 +2038,6 @@ bool scaleTest()
 	//---
 
 
-	//OK, so now do the rotation
 	//Do the refresh
 	ProgressData p;
 	TEST(!f->refresh(streamIn,streamOut,p,dummyCallback),"refresh error code");
@@ -2056,8 +2052,8 @@ bool scaleTest()
 	//Scaling around its centre of mass
 	// should scale the bounding box by the cube of the scale factor
 	BoundCube bc[2];
-	bc[0]=getIonDataLimits(d->data);
-	bc[1]=getIonDataLimits(outData->data);
+	IonHit::getBoundCube(d->data,bc[0]);
+	IonHit::getBoundCube(outData->data,bc[1]);
 
 	float cubeOfScale=scaleFact*scaleFact*scaleFact;
 
@@ -2115,7 +2111,6 @@ bool scaleAnisoTest()
 	//---
 
 
-	//OK, so now do the rotation
 	//Do the refresh
 	ProgressData p;
 	TEST(!f->refresh(streamIn,streamOut,p,dummyCallback),"refresh error code");
