@@ -22,6 +22,8 @@
 
 #include "backend/filter.h"
 
+#include <map>
+
 
 #if  defined(WIN32) || defined(WIN64)
 	//Help mathgl out a bit: we don't need the GSL on this platform
@@ -40,7 +42,6 @@
 #undef isnan
 
 
-
 enum
 {
 	PLOT_TYPE_ONED,
@@ -63,7 +64,7 @@ unsigned int plotID(const std::string &plotString);
 //!Return the error mode type, given the human readable string
 unsigned int plotErrmodeID(const std::string &s);
 		
-//!Nasty hack class to change mathgl API from named char pallette to rgb specification
+//!Nasty hack class to change mathgl API from named char palette to rgb specification
 class MGLColourFixer
 {
 	private:
@@ -77,30 +78,137 @@ class MGLColourFixer
 		//Get the best colour that is available
 		// returns the char to give to mathgl; may be exact,
 		// maybe nearest match, depending upon number of colours used
-		// and mgl pallette size
+		// and mgl palette size
 		char getNextBestColour(float r, float g, float b);
 
 		static unsigned int getMaxColours();
 };
 
+
 //!Data class  for holding info about non-overlapping 
 // interactive rectilinear "zones" overlaid on plots 
+// update method must be set.
 class PlotRegion
 {
+	private:
+		size_t accessMode;
+
+		void *parentObject;
+
+
 	public:
-		//Axis along which bounds are set. Each entry is unique
-		vector<int> boundAxis;
+		enum
+		{
+			ACCESS_MODE_FILTER,
+			ACCESS_MODE_RANGEFILE,
+			ACCESS_MODE_ENUM_END
+		};
+		
 		//Bounding limits for axial bind
 		vector<std::pair<float,float> > bounds;
 
 		//Bounding region colour
 		float r,g,b;
-		//The parent filter, so region can tell who the parent is
-		Filter *parentFilter;
-		//The ID value for this region, used when interacting with parent filter
+
+		//The ID value for this region, used when interacting with parent object
 		unsigned int id;
-		//!Unique ID for region across entire multiplot
-		unsigned int uniqueID;
+		
+		PlotRegion();
+
+		PlotRegion(size_t updateMethod,void *parentObject);
+
+		const PlotRegion &operator=(const PlotRegion &r);
+
+		//Alter the update method
+		void setUpdateMethod(size_t updateMethod, void *parentObject);
+
+		//Update the parent object using the curretn update method
+		void updateParent(size_t method, const vector<float> &newBounds, bool updateSelf=true);
+
+		//Retrieve the parent as a filter object - must be in ACCESS_MODE_FILTER
+		Filter *getParentAsFilter() const { ASSERT(accessMode==ACCESS_MODE_FILTER); return (Filter*)parentObject;};
+
+		RangeFile *getParentAsRangeFile() const { ASSERT(accessMode==ACCESS_MODE_RANGEFILE); return (RangeFile*)parentObject;};
+
+};
+
+//Handles an array of regions, for drawing and editing of the array
+
+class RegionGroup
+{
+	private:
+		//cache for region overlaps, to reduce need to search
+		mutable vector<pair<size_t,size_t> > overlapIdCache;
+		mutable vector<pair<float,float> > overlapCoordsCache;
+
+		mutable bool haveOverlapCache;
+	public:
+		RegionGroup() { haveOverlapCache=false;};
+		//!Interactive, or otherwise marked plot regions
+		vector<PlotRegion> regions;
+
+
+		void clear() {regions.clear(); };
+		
+		//!Append a region to the plot
+		void addRegion(unsigned int regionId, float start, float end,	
+				float r,float g, float b, Filter *parentFilter);
+		//!Retrieve the ID of the non-overlapping region in X-Y space
+		bool getRegionIdAtPosition(float x, float y, unsigned int &id) const;
+		//!Retrieve a region using its ID
+		void getRegion(unsigned int id, PlotRegion &r) const;
+
+		//!Get the number of regions;
+		size_t getNumRegions() const { return regions.size();};
+		
+		//!Pass the region movement information to the parent filter object
+		// newX and newY should be absolute positions.
+		void moveRegion(unsigned int regionId, unsigned int method, bool selfUpdate,
+							float newX, float newY);
+		//!Obtain limit of motion for a given region movement type
+		void findRegionLimit(unsigned int regionId,
+				unsigned int movementType, float &maxX, float &maxY) const;
+
+
+		void getOverlaps(vector<pair<size_t,size_t> > &ids,
+				vector< pair<float,float> > &coords) const;
+};
+
+struct  OVERLAY_DATA
+{
+	//Coordinate and amplitude
+	vector<pair<float, float> > coordData;
+	//title for all of te specified overlay data
+	string title;
+	//If the overlay is enabled or not
+	bool enabled;
+};
+
+class PlotOverlays
+{
+	private:
+		bool isEnabled;
+		// List of the overlays that can be shown on the given plot
+		vector<OVERLAY_DATA>  overlayData;
+	public:
+		PlotOverlays() : isEnabled(true) {}
+		//Add a new overlay to the plot
+		void add(const OVERLAY_DATA &overlay) {overlayData.push_back(overlay);}
+		//Draw the overlay on the current plot
+		void draw(mglGraph *g,MGLColourFixer &fixer, 
+			const mglPoint &boundMin, const mglPoint &boundMax,bool logMode) const;
+		//Enable the specified overlay
+		void setEnabled(size_t offset,bool isEnabled) 
+			{ASSERT(offset <overlayData.size()); overlayData[offset].enabled=isEnabled;}
+
+		void setEnabled(bool doEnable)  {isEnabled=doEnable;}
+		bool enabled() const { return isEnabled;}
+		
+		void clear() {overlayData.clear();}
+		void erase(size_t item) {ASSERT(item < overlayData.size()); overlayData.erase(overlayData.begin() + item);;}
+
+		const vector<OVERLAY_DATA> &getOverlays() const { return overlayData;};
+
 };
 
 //!Base class for data plotting
@@ -109,6 +217,9 @@ class PlotBase
 	public:
 		PlotBase(){};
 		virtual ~PlotBase(){};
+
+		virtual PlotBase *clone() const = 0;
+
 		//The type of plot (ie what class is it?)	
 		unsigned int plotType;
 		
@@ -135,15 +246,17 @@ class PlotBase
 		//!Pointer to some constant object that generated this plot
 		const void *parentObject;
 
-		//!Interactive, or otherwise marked plot regions
-		vector<PlotRegion> regions;
-		//ID handler for regions
-		UniqueIDHandler regionIDHandler;
-
 		//!integer to show which of the n plots that the parent generated
 		//that this data is represented by
 		unsigned int parentPlotIndex;
-		
+
+
+		RegionGroup regionGroup;
+
+	
+		//True if the plot has data
+		virtual bool empty() const=0;
+
 		//Draw the plot onto a given MGL graph
 		virtual void drawPlot(mglGraph *graph, MGLColourFixer &fixer) const=0;
 
@@ -155,21 +268,6 @@ class PlotBase
 		virtual void getRawData(vector<vector<float> > &f,
 				std::vector<std::wstring> &labels) const=0;
 
-		//!Retrieve the ID of the non-overlapping region in X-Y space
-		virtual bool getRegionIdAtPosition(float x, float y, unsigned int &id) const=0;
-		//!Retrieve a region using its unique ID
-		virtual void getRegion(unsigned int id, PlotRegion &r) const=0;
-
-		//!Get the number of regions;
-		size_t getNumRegions() const { return regions.size();};
-		
-		//!Pass the region movement information to the parent filter object
-		virtual void moveRegion(unsigned int regionId, unsigned int method, 
-							float newX, float newY) const=0;
-
-		//!Obtain limit of motion for a given region movement type
-		virtual void moveRegionLimit(unsigned int regionId,
-				unsigned int movementType, float &maxX, float &maxY) const=0;
 
 };
 
@@ -184,11 +282,11 @@ class Plot1D : public PlotBase
 		//!Data
 		std::vector<float> xValues,yValues,errBars;
 		
-		
 	public:
 		Plot1D();
-		
-		
+		virtual bool empty() const;
+		virtual PlotBase *clone() const;
+			
 		void getBounds(float &xMin,float &xMax,float &yMin,float &yMax) const;
 
 		//!Set the plot data from a pair and symmetric Y error
@@ -199,16 +297,8 @@ class Plot1D : public PlotBase
 		void setData(const vector<float> &vX, const vector<float> &vY,
 							const vector<float> &symYErr);
 
-		//!Append a region to the plot
-		void addRegion(unsigned int parentPlot, unsigned int regionId,
-			       		float start, float end,	float r,float g, 
-						float b, Filter *parentFilter);
 		
-		//!Try to move a region from its current position to a new position
-		//return the test coord. valid methods are 0 (left extend), 1 (slide), 2 (right extend)
-		float moveRegionTest(unsigned int region, unsigned int method, float newPos) const;
-
-		//!Move a region to a new location. MUST call moveRegionTest first.
+		//!Move a region to a new location. 
 		void moveRegion(unsigned int region, unsigned int method, float newPos);
 
 		void clear(bool preserveVisibility);
@@ -230,7 +320,7 @@ class Plot1D : public PlotBase
 		//!Retrieve the ID of the non-overlapping region in X-Y space
 		bool getRegionIdAtPosition(float x, float y, unsigned int &id) const;
 
-		//!Retrieve a region using its unique ID
+		//!Retrieve a region using its ID
 		void getRegion(unsigned int id, PlotRegion &r) const;
 	
 		//!Pass the region movement information to the parent filter object
@@ -240,7 +330,7 @@ class Plot1D : public PlotBase
 		//overlap after a move operation. Note that the output variables will only
 		//be set for the appropriate motion direction. Eg, an X only move will not
 		//set limitY.
-		void moveRegionLimit(unsigned int regionId,
+		void findRegionLimit(unsigned int regionId,
 				unsigned int movementType, float &limitX, float &limitY) const;
 
 		bool wantLogPlot() const { return logarithmic;};
@@ -256,7 +346,8 @@ class PlotWrapper
 		//!Elements of the plot
 		std::vector<PlotBase *> plottingData;
 
-		//!which plots were last visible?
+		//! Data regarding plots were visible previously
+		// first pair entry is the parent object pointer. second is the parent plot index
 		std::vector<std::pair< const void *, unsigned int> > lastVisiblePlots;
 	
 		//Position independant ID handling for the 
@@ -279,16 +370,40 @@ class PlotWrapper
 		//  will be hinted to take correct action
 		bool interactionLocked;
 
+		//!Do we want to highlight positions where regions overlap?
+		bool highlightRegionOverlaps;
+
 	public:
+		//"stick" type overlays for marking amplitudes on top of the plot
+		PlotOverlays overlays;
+		
 		//!Constructor
 		PlotWrapper();
 
 		//!Destructor must delete target plots
 		~PlotWrapper();
 
+		const PlotWrapper &operator=(const PlotWrapper &oth);
 
+		//Return  the number of plots
+		size_t numPlots() const { return plottingData.size();}
+
+		//Retrieve the IDs for the stored plots
+		void getPlotIDs(vector<unsigned int> &ids) const ;
+
+		//Retrieve the title of the plot
+		std::wstring getTitle(size_t plotId) const;
+
+
+		void setEnableHighlightOverlap(bool enable=true) { highlightRegionOverlaps=enable;}
+
+		//get the type of parent filter that generated the plot
+		size_t getParentType(size_t plotId) const;
+
+		//True if user is not allowed to interact with plot
 		bool isInteractionLocked() const { return interactionLocked;};
 
+		//Disallow interaction with plot (lock=true), or enable interaction (lock=false)
 		void lockInteraction(bool lock=true) {interactionLocked=lock;};
 
 		//!Has the contents of the plot changed since the last call to resetChange?
@@ -318,7 +433,10 @@ class PlotWrapper
 		//!Set the parent information for a given plot
 		void setParentData(unsigned int plotID,
 				const void *parentObj, unsigned int plotIndex);
-		
+	
+		//!Get the parent object fo rthis plot
+		const void *getParent(unsigned int plotID) { ASSERT(plotID < plottingData.size()); return plottingData[plotID]->parentObject;}
+
 		//!Set the plotting mode.
 		void setTraceStyle(unsigned int plotID,unsigned int mode);
 
@@ -336,6 +454,9 @@ class PlotWrapper
 
 		//!Get the number of visible plots
 		unsigned int getNumVisible() const;
+		
+		//!Get the number of visible plots
+		unsigned int getNumTotal() const { return plottingData.size(); };
 
 		//!Returns true if plot is visible, based upon its uniqueID.
 		bool isPlotVisible(unsigned int plotID) const;
@@ -368,6 +489,14 @@ class PlotWrapper
 		//Return the region data for the given regionID/plotID combo
 		void getRegion(unsigned int plotId, unsigned int regionId, PlotRegion &r) const;
 
+		//Get all of the (id, regions) for plots. Bool allows for only the plots that are visible to be obtained
+		void getRegions(vector<pair<size_t,vector<PlotRegion> > > &regions, bool visibleOnly=true) const;
+		
+		//Return the ID and coordinates of any overlapping regions
+		// - this only returns overlaps for individual plots - not between plots
+		void getRegionOverlaps(std::vector<pair<size_t,size_t> > &ids,
+							std::vector< pair<float,float> > &coords) const;
+
 		//!Retrieve the raw data associated with the selected plots.
 		void getRawData(vector<vector<vector<float> > >  &data, std::vector<std::vector<std::wstring> >  &labels) const;
 	
@@ -379,12 +508,18 @@ class PlotWrapper
 		unsigned int getVisibleType() const;
 
 		//!Obtain limit of motion for a given region movement type
-		void moveRegionLimit(unsigned int plotId, unsigned int regionId,
+		void findRegionLimit(unsigned int plotId, unsigned int regionId,
 				unsigned int movementType, float &maxX, float &maxY) const;
 
 		//!Pass the region movement information to the parent filter object
-		void moveRegion(unsigned int plotID, unsigned int regionId, unsigned int movementType, 
-							float newX, float newY) const;
+		void moveRegion(unsigned int plotID, unsigned int regionId, bool regionSelfUp,
+					unsigned int movementType, float newX, float newY) const;
+
+		//Change the regions to modify the given rangefiles, for each of the 
+		// rangefile filters in the map
+		void switchOutRegionParent(std::map<const RangeFileFilter *, RangeFile> &switchMap);
+
+		void setRegionGroup(size_t plotId, RegionGroup &r);
 };
 
 #endif
