@@ -255,6 +255,48 @@ bool Point3D::orthogonalise(const Point3D &pt)
 	return true;
 }
 
+Point3D Point3D::centroid(const Point3D *p, unsigned int n)
+{
+	ASSERT(p);
+	Point3D centroid(0,0,0);
+
+	/* This code should work, but is not profiled. No caller uses
+	   this code at time of writing.
+#ifdef _OPENMP
+
+	//Parallel version
+	//--
+	vector<Point3D> centroids(omp_get_max_threads(),Point3D(0,0,0));
+#pragma omp parallel for 
+	for(size_t ui=0;ui<n;ui++)
+		centroids[omp_get_thread_num()]+=p[ui];
+
+	for(size_t ui=0;ui<centroids.size();ui++)
+		centroid+=centroids[ui];
+	//--
+#endif
+	*/
+	for(unsigned int ui=0;ui<n;ui++)
+		centroid+=p[ui];
+
+	centroid*=1.0f/(float)n;
+
+	return centroid;
+}
+
+Point3D Point3D::centroid(const std::vector<Point3D> &p)
+{
+	Point3D centroid(0,0,0);
+
+	for(unsigned int ui=0;ui<p.size();ui++)
+		centroid+=p[ui];
+
+	centroid*=1.0f/(float)p.size();
+
+	return centroid;
+}
+
+
 bool Point3D::parse(const std::string &str)
 {
 	//Needs to be at minimum #,#,#
@@ -509,7 +551,7 @@ void quat_pointmult(Point3f *result, const Quaternion *q1, const Quaternion *q2)
 //failure to do so will have weird results. 
 //For better performance on multiple rotations, use other function
 //Note result is stored in returned point
-void quat_rot(Point3f *point, Point3f *rotVec, float angle)
+void quat_rot(Point3f *point, const Point3f *rotVec, float angle)
 {
 	ASSERT(rotVec->fx*rotVec->fx + rotVec->fy*rotVec->fy + rotVec->fz*rotVec->fz - 1.0f < 
 			5.0f*sqrt(std::numeric_limits<float>::epsilon()));
@@ -548,8 +590,8 @@ void quat_rot(Point3f *point, Point3f *rotVec, float angle)
 }
 
 
-//Retrieve the quaternion for repeated rotation. pass to the quat_rot_apply_quats
-void quat_get_rot_quat(Point3f *rotVec, float angle,Quaternion *rotQuat) 
+//Retrieve the quaternion for repeated rotation. Pass to the quat_rot_apply_quats
+void quat_get_rot_quat(const Point3f *rotVec, float angle,Quaternion *rotQuat) 
 {
 	ASSERT(rotVec->fx*rotVec->fx + rotVec->fy*rotVec->fy + rotVec->fz*rotVec->fz - 1.0f < 
 			5.0f*sqrt(std::numeric_limits<float>::epsilon()));
