@@ -20,6 +20,8 @@
 
 #include "common/basics.h"
 
+#include <sys/time.h>
+
 using std::string;
 using std::vector;
 
@@ -123,6 +125,21 @@ bool isVersionNumberString(const std::string &s)
 }
 
 
+bool boolStrDec(const std::string &s, bool &b)
+{
+	std::string tmp;
+	tmp=stripWhite(s);
+
+	if(tmp == "0")
+		b=false;
+	else if(tmp == "1")
+		b=true;
+	else 
+		return false; // Failed to decode
+
+	return true;
+}
+
 std::string onlyFilename( const std::string& path) 
 {
 #if defined(_WIN32) || defined(_WIN64)
@@ -191,7 +208,7 @@ bool genRandomFilename(std::string &s,bool seedRand)
 
 	//some valid chars for generating random strings
 	static const char validChars[] =
-	    "0123456789_-"
+	    "0123456789_"
 	    "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
 	    "abcdefghijklmnopqrstuvwxyz";
 
@@ -387,6 +404,11 @@ void genColString(unsigned char r, unsigned char g,
 
 //Strip "whitespace"
 std::string stripWhite(const std::string &str)
+{
+	return stripChars(str,"\f\n\r\t ");
+}
+
+std::string stripChars(const std::string &str, const char *chars)
 {
 	using std::string;
 
@@ -395,8 +417,8 @@ std::string stripWhite(const std::string &str)
 	if(!str.size())
 	      return str;
 
- 	start = str.find_first_not_of("\f\n\r\t ");
-	end = str.find_last_not_of("\f\n\r\t ");
+ 	start = str.find_first_not_of(chars);
+	end = str.find_last_not_of(chars);
 	if(start == string::npos) 
 		return string("");
 	else
