@@ -16,59 +16,119 @@
  *	along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+#if defined(WIN32) || defined(WIN64)
+#include <GL/glew.h>
+#endif
+
 #include "scene.h"
 
-
+#include "./backend/viscontrol.h"
+#include "./backend/filter.h"
 
 using std::vector;
 
-Scene::Scene() : tempCam(0), activeCam(0), cameraSet(false), outWinAspect(1.0f), r(0.0f), g(0.0f), b(0.0f)
+
+//
+const char ANIMATE_PROGRESS_BASENAME[] = {
+	"textures/animProgress"};
+unsigned int ANIMATE_PROGRESS_NUMFRAMES=3;
+
+
+
+Scene::Scene() : tempCam(0), cameraSet(true), outWinAspect(1.0f)
 {
+	glewInited=false;
+
 	lastHovered=lastSelected=(unsigned int)(-1);
 	lockInteract=false;
 	hoverMode=selectionMode=false;
-	viewRestrict=false;
 	useAlpha=true;
 	useLighting=true;
 	useEffects=false;
 	showAxis=true;
+	attemptedLoadProgressAnim=false;
+	showProgressAnimation=false;
 
 	//default to black
 	rBack=gBack=bBack=0.0f;
 
+	activeCam = new CameraLookAt;
+
+	lightPosition[0]= 1.0;
+	lightPosition[1]= 1.0;
+	lightPosition[2]= 1.0;
+	lightPosition[3]=0.0;
+
+
+	DrawableObj::setTexPool(new TexturePool);
+
 }
 
 Scene::~Scene()
 {
 	clearAll();
+	DrawableObj::clearTexPool();
 }
 
 unsigned int Scene::initDraw()
 {
+	//Initialise GLEW
+#if defined(WIN32) || defined(WIN64)
+	if(!glewInited)
+	{
+		unsigned int errCode;
+		errCode=glewInit();
+		if(errCode!= GLEW_OK)
+		{
+			cerr << "Opengl context could not be created, aborting." << endl;
+			cerr << "Glew reports:" << glewGetErrorString(errCode) << endl;
+			//Blow up without opengl
+			exit(1);
+		}
+		glewInited=true;
+	}
+#endif
+	glClearColor( rBack, gBack, bBack,1.0f );
 	glClear(GL_COLOR_BUFFER_BIT |GL_DEPTH_BUFFER_BIT);
 
+	glEnable(GL_CULL_FACE);
+	glCullFace(GL_BACK);
+
+	glEnable(GL_DEPTH_TEST);
+
+	glEnable(GL_COLOR_MATERIAL);
+	glColorMaterial ( GL_FRONT, GL_AMBIENT_AND_DIFFUSE ) ;
+
+	glEnable(GL_POINT_SMOOTH);
+	glEnable(GL_LINE_SMOOTH);
+	
+	//Will it blend? That is the question...
+	// let the objects know about this, so they
+	// can pick the right algorithm
+	DrawableObj::setUseAlphaBlending(useAlpha);
+	
+	glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
+	
 	if(useAlpha)
 		glEnable(GL_BLEND);
 	else
 		glDisable(GL_BLEND);
 
 	glDisable(GL_LIGHTING);
+	glEnable(GL_LIGHT0);
+	glShadeModel(GL_SMOOTH);
 	//Set up the scene lights
 	//==
+
 	//Set up default lighting
 	const float light_ambient[] = { 0.0, 0.0, 0.0, 1.0 };
 	const float light_diffuse[] = { 1.0, 1.0, 1.0, 1.0 };
-	const float light_specular[] = { 0.0, 0.0, 0.0, 0.0 };
-	float light_position[] = { 1.0, 1.0, 1.0, 0.0 };
 	glLightfv(GL_LIGHT0, GL_AMBIENT, light_ambient);
 	glLightfv(GL_LIGHT0, GL_DIFFUSE, light_diffuse);
-	glLightfv(GL_LIGHT0, GL_SPECULAR, light_specular);
-	glLightfv(GL_LIGHT0, GL_POSITION, light_position);
+	// The direction the light shines in
+	glLightfv(GL_LIGHT0, GL_POSITION, lightPosition); 
 
 
-
-
-	glDisable(GL_LIGHTING);
 	//==
 	
 	//Use the active if set 
@@ -79,7 +139,27 @@ unsigned int Scene::initDraw()
 
 	}
 
+	//Attempt to load the progress animation, if we have not tried before
+	if(!attemptedLoadProgressAnim)
+	{
+		attemptedLoadProgressAnim=true;
 
+		//Load the animation textures
+		vector<string> animFiles;
+		animFiles.resize(ANIMATE_PROGRESS_NUMFRAMES);
+		string animFilename,tmpStr;
+		for(size_t ui=0;ui<ANIMATE_PROGRESS_NUMFRAMES;ui++)
+		{
+			animFilename=ANIMATE_PROGRESS_BASENAME;
+			stream_cast(tmpStr,ui);
+			animFilename+=tmpStr + string(".png");
+			animFiles[ui]=animFilename;
+		}
+
+		progressAnimTex.setTexture(animFiles);
+
+		updateProgressOverlay();
+	}
 
 	//Let the effects objects know about the scene
 	Effect::setBoundingCube(boundCube);
@@ -95,77 +175,73 @@ unsigned int Scene::initDraw()
 	return passes;
 }
 
-void Scene::updateCam(const Camera *camToUse) const
+void Scene::updateCam(const Camera *camToUse, bool useIdent=true) const
 {
 	Point3D lightNormal;
 	
-
-	glLoadIdentity();
-	//If viewport restriction is on, inform camera to
-	//shrink viewport to specified region
-	if(viewRestrict)
-	{
-		camToUse->apply(outWinAspect,boundCube,true,
-				viewRestrictStart[0],viewRestrictEnd[0],
-				viewRestrictStart[1],viewRestrictEnd[1]);
-	}
-	else
-		camToUse->apply(outWinAspect,boundCube);
+	camToUse->apply(outWinAspect,boundCube,useIdent);
 
 	lightNormal=camToUse->getViewDirection();
 	glNormal3f(lightNormal[0],lightNormal[1],lightNormal[2]);	
-
 }
 
-void Scene::draw() 
+void Scene::updateProgressOverlay()
+{
+	progressAnimTex.setPos(0.9*winX,0.9*winY);
+	progressAnimTex.setSize(0.1*winX);
+	//Cycle every this many seconds
+	progressAnimTex.setRepeatTime(6.0f);
+	//Ramp opacity for this long (seconds)
+	progressAnimTex.setFadeInTime(1.0f);
+	//Don't show the animation until this many sceonds
+	progressAnimTex.setShowDelayTime(1.0f);
+}
+
+void Scene::draw(bool noUpdateCam) 
 {
 
 	glPushMatrix();
 
-
 	Camera *camToUse;
 	if(tempCam)
 		camToUse=tempCam;
 	else
-	{
-		ASSERT(activeCam < cameras.size());
-		camToUse=cameras[activeCam];
-	}
-	//Inform text about current camera, so it can billboard if needed
+		camToUse=activeCam;
+
+	//Inform text about current camera, 
+	// so it can eg billboard if needed
 	DrawableObj::setCurCamera(camToUse);
+	DrawableObj::setWindowSize(winX,winY);
+	DrawableObj::setBackgroundColour(rBack,gBack,bBack);
 	Effect::setCurCam(camToUse);
 
 
 	bool lightsOn=false;
+	//Find number of passes to  perform
 	unsigned int numberTotalPasses;
 	numberTotalPasses=initDraw();
 
-	unsigned int passNumber=0;
-
-	if(cameraSet)
+	if(cameraSet && !noUpdateCam)
 		updateCam(camToUse);
 
 
 
-	bool needCamUpdate=false;
-	while(passNumber < numberTotalPasses)
+	for(unsigned int passNumber=0; passNumber<numberTotalPasses; passNumber++)
 	{
 
 		if(useEffects)
 		{
+			bool needCamUpdate=false;
 			for(unsigned int ui=0;ui<effects.size();ui++)
 			{
 				effects[ui]->enable(passNumber);
 				needCamUpdate|=effects[ui]->needCamUpdate();
 			}
 
-			if(cameraSet && needCamUpdate)
-			{
-				glLoadIdentity();	
+			if(cameraSet && !noUpdateCam && needCamUpdate )
 				updateCam(camToUse);
-			}
 		}
-
+		
 
 		if(showAxis)
 		{
@@ -190,7 +266,7 @@ void Scene::draw()
 		drawObjectVector(objects,lightsOn,true);
 		//-----------	
 		
-		//Second pass with transparent objects
+		//Second sub-pass with transparent objects
 		//-----------	
 		//Draw the referenced objects
 		drawObjectVector(refObjects,lightsOn,false);
@@ -199,8 +275,6 @@ void Scene::draw()
 		//-----------	
 		
 		
-		glFlush();
-		passNumber++;
 	}
 	
 	
@@ -224,6 +298,9 @@ void Scene::draw()
 		drawHoverOverlay();
 	drawOverlays();
 
+	//Draw progress, if needed
+	drawProgressAnim();
+
 }
 
 void Scene::drawObjectVector(const vector<const DrawableObj*> &drawObjs, bool &lightsOn, bool drawOpaques) const
@@ -286,17 +363,7 @@ void Scene::drawOverlays() const
 
 	glDisable(GL_LIGHTING);
 	//Set the opengl camera state back into modelview mode
-	if(viewRestrict)
-	{
-
-		//FIXME: How does the aspect ratio fit in here?
-		gluOrtho2D(viewRestrictStart[0],
-				viewRestrictEnd[0],
-				viewRestrictStart[0],
-				viewRestrictStart[1]);
-	}
-	else
-		gluOrtho2D(0, outWinAspect, 1.0, 0);
+	gluOrtho2D(0, outWinAspect, 1.0, 0);
 
 
 
@@ -365,16 +432,8 @@ void Scene::drawHoverOverlay()
 
 
 		const float ICON_SIZE= 0.05;
-		binderIcons.setTexturePool(&texPool);
-		binderIcons.setWindowSize(winX,winY);	
 		binderIcons.setSize(ICON_SIZE*winY);
-		
-		mouseIcons.setTexturePool(&texPool);
-		mouseIcons.setWindowSize(winX,winY);	
 		mouseIcons.setSize(ICON_SIZE*winY);
-	
-		keyIcons.setTexturePool(&texPool);
-		keyIcons.setWindowSize(winX,winY);	
 		keyIcons.setSize(ICON_SIZE*winY);
 
 		unsigned int iconNum=0;
@@ -388,14 +447,14 @@ void Scene::drawHoverOverlay()
 				case BIND_MODE_FLOAT_SCALE:
 				case BIND_MODE_FLOAT_TRANSLATE:
 				case BIND_MODE_POINT3D_SCALE:
-					foundIconTex=binderIcons.setTexture(TEST_OVERLAY_PNG[TEXTURE_ENLARGE]);
+					foundIconTex=binderIcons.setTexture(TEXTURE_OVERLAY_PNG[TEXTURE_ENLARGE]);
 					break;
 				case BIND_MODE_POINT3D_TRANSLATE:
-					foundIconTex=binderIcons.setTexture(TEST_OVERLAY_PNG[TEXTURE_TRANSLATE]);
+					foundIconTex=binderIcons.setTexture(TEXTURE_OVERLAY_PNG[TEXTURE_TRANSLATE]);
 					break;
 				case BIND_MODE_POINT3D_ROTATE:
 				case BIND_MODE_POINT3D_ROTATE_LOCK:
-					foundIconTex=binderIcons.setTexture(TEST_OVERLAY_PNG[TEXTURE_ROTATE]);
+					foundIconTex=binderIcons.setTexture(TEXTURE_OVERLAY_PNG[TEXTURE_ROTATE]);
 				default:
 					break;
 			}
@@ -404,13 +463,13 @@ void Scene::drawHoverOverlay()
 			switch(binder[ui]->getMouseButtons())
 			{
 				case SELECT_BUTTON_LEFT:
-					foundMouseTex=mouseIcons.setTexture(TEST_OVERLAY_PNG[TEXTURE_LEFT_CLICK]);
+					foundMouseTex=mouseIcons.setTexture(TEXTURE_OVERLAY_PNG[TEXTURE_LEFT_CLICK]);
 					break;
 				case SELECT_BUTTON_MIDDLE:
-					foundMouseTex=mouseIcons.setTexture(TEST_OVERLAY_PNG[TEXTURE_MIDDLE_CLICK]);
+					foundMouseTex=mouseIcons.setTexture(TEXTURE_OVERLAY_PNG[TEXTURE_MIDDLE_CLICK]);
 					break;
 				case SELECT_BUTTON_RIGHT:
-					foundMouseTex=mouseIcons.setTexture(TEST_OVERLAY_PNG[TEXTURE_RIGHT_CLICK]);
+					foundMouseTex=mouseIcons.setTexture(TEXTURE_OVERLAY_PNG[TEXTURE_RIGHT_CLICK]);
 					break;
 				default:
 					//The flags are or'd together, so we can get other combinations
@@ -424,13 +483,13 @@ void Scene::drawHoverOverlay()
 			{
 				case FLAG_CMD:
 #ifdef __APPLE__
-					foundKeyTex=keyIcons.setTexture(TEST_OVERLAY_PNG[TEXTURE_COMMAND]);
+					foundKeyTex=keyIcons.setTexture(TEXTURE_OVERLAY_PNG[TEXTURE_COMMAND]);
 #else
-					foundKeyTex=keyIcons.setTexture(TEST_OVERLAY_PNG[TEXTURE_CTRL]);
+					foundKeyTex=keyIcons.setTexture(TEXTURE_OVERLAY_PNG[TEXTURE_CTRL]);
 #endif
 					break;
 				case FLAG_SHIFT:
-					foundKeyTex=keyIcons.setTexture(TEST_OVERLAY_PNG[TEXTURE_SHIFT]);
+					foundKeyTex=keyIcons.setTexture(TEXTURE_OVERLAY_PNG[TEXTURE_SHIFT]);
 					break;
 				default:
 					//The flags are or'd together, so we can get other combinations
@@ -473,11 +532,21 @@ void Scene::drawHoverOverlay()
 	glEnable(GL_DEPTH_TEST);
 }
 
+void Scene::drawProgressAnim() const
+{
+	if(!showProgressAnimation)
+		return;
+
+	if(!progressAnimTex.isOK())
+		return;
+
+	progressAnimTex.draw();
+}
 
 void Scene::commitTempCam()
 {
 	ASSERT(tempCam);
-	std::swap(cameras[activeCam], tempCam);
+	std::swap(activeCam, tempCam);
 	delete tempCam;
 	tempCam=0;
 }
@@ -493,9 +562,9 @@ void Scene::setTempCam()
 	//If a temporary camera is not set, set one.
 	//if it is set, update it from the active camera
 	if(!tempCam)
-		tempCam =cameras[activeCam]->clone();
+		tempCam =activeCam->clone();
 	else
-		*tempCam=*cameras[activeCam];
+		*tempCam=*activeCam;
 }
 
 void Scene::addDrawable(DrawableObj const *obj )
@@ -526,7 +595,6 @@ void Scene::clearAll()
 	clearObjs();
 	clearRefObjs();
 	clearBindings();	
-	clearCams();
 }
 
 void Scene::clearObjs()
@@ -545,47 +613,27 @@ void Scene::clearBindings()
 }
 
 
-void Scene::clearCams()
-{
-	for(unsigned int ui=0; ui<cameras.size(); ui++)
-		delete cameras[ui];
-
-	cameras.clear();
-	camIDs.clear();
-}
-
 void Scene::clearRefObjs()
 {
 	refObjects.clear();
 }
 
-
-unsigned int Scene::addCam(Camera *c)
-{
-	ASSERT(c);
-	ASSERT(cameras.size() == camIDs.size());
-	cameras.push_back(c);
-	unsigned int id = camIDs.genId(cameras.size()-1);
-
-	if(cameras.size() == 1)
-		setDefaultCam();
-	return id;
+void Scene::getLightPos(float *f) const
+{ 
+	for(unsigned int ui=0;ui<4;ui++)
+		f[ui] = lightPosition[ui];
 }
 
-void Scene::removeCam(unsigned int camIDVal)
+void Scene::setLightPos(const float *f)
 {
-	unsigned int position = camIDs.getPos(camIDVal);
-	delete cameras[position];
-	cameras.erase(cameras.begin()+position);
-	camIDs.killByPos(position);
+	for(unsigned int ui=0;ui<4;ui++)
+		lightPosition[ui]=f[ui];
+}
 
-	if(cameras.size())
-	{
-		activeCam=0;
-		cameraSet=true;
-	}
-	else
-		cameraSet=false;
+void Scene::setShowProgress(bool show)
+{
+	showProgressAnimation=show;
+	progressAnimTex.resetTime();
 }
 
 
@@ -594,37 +642,12 @@ void Scene::setAspect(float newAspect)
 	outWinAspect=newAspect;
 }
 
-void Scene::setActiveCam(unsigned int idValue)
+void Scene::setActiveCamByClone(const Camera *c)
 {
-
 	if(tempCam)
 		discardTempCam();
 
-	unsigned int position = camIDs.getPos(idValue);
-	activeCam = position;
-	cameraSet=true;
-}
-
-void Scene::setDefaultCam(Camera *newCam, bool setAsActive)
-{
-	ASSERT(cameras.size() || newCam);
-
-	//If the scene is using a temporary camera
-	// (eg for animation or visual effects), discard it
-	if(tempCam)
-		discardTempCam();
-
-	//If we don't have a default camera, set one
-	if(cameras.size() && newCam)
-	{
-		if(newCam)
-			cameras[0] = newCam;
-	}
-	else if(newCam)
-		cameras.push_back(newCam);
-
-	if(setAsActive)
-		activeCam=0;
+	activeCam = c->clone();
 	cameraSet=true;
 }
 
@@ -633,7 +656,7 @@ void Scene::ensureVisible(unsigned int direction)
 {
 	computeSceneLimits();
 
-	cameras[activeCam]->ensureVisible(boundCube,direction);
+	activeCam->ensureVisible(boundCube,direction);
 }
 
 void Scene::computeSceneLimits()
@@ -668,7 +691,7 @@ void Scene::computeSceneLimits()
 		boundCube.setBounds(-0.5,-0.5,-0.5,
 					0.5,0.5,0.5);
 	}
-	//Now that we have a scene level bounding box,
+	//NOw that we have a scene level bounding box,
 	//we need to set the camera to ensure that
 	//this box is visible
 	ASSERT(boundCube.isValid());
@@ -687,8 +710,7 @@ void Scene::computeSceneLimits()
 
 Camera *Scene::getActiveCam()
 {
-	ASSERT(cameras.size());
-	return cameras[activeCam];
+	return activeCam;
 }
 
 Camera *Scene::getTempCam()
@@ -697,32 +719,6 @@ Camera *Scene::getTempCam()
 	return tempCam;
 }
 
-void Scene::getCamProperties(unsigned int uniqueID, CameraProperties &p) const
-{
-	unsigned int position=camIDs.getPos(uniqueID);
-	cameras[position]->getProperties(p);
-}
-
-void Scene::getCameraIDs(vector<std::pair<unsigned int, std::string> > &idVec) const
-{
-	std::vector<unsigned int> ids;
-	camIDs.getIds(ids);
-
-	idVec.resize(ids.size());
-	for(unsigned int ui=0;ui<ids.size(); ui++)
-	{
-		idVec[ui] = std::make_pair(ids[ui],
-			cameras[camIDs.getPos(ids[ui])]->getUserString());
-	}
-}
-
-bool Scene::setCamProperty(unsigned int uniqueID, unsigned int key,
-	       					const std::string &value) 
-{
-	unsigned int position=camIDs.getPos(uniqueID);
-	return cameras[position]->setProperty(key,value);
-}
-
 //Adapted from 
 //http://chadweisshaar.com/robotics/docs/html/_v_canvas_8cpp-source.html
 //GPLv3+ permission obtained by email inquiry.
@@ -734,7 +730,6 @@ unsigned int Scene::glSelect(bool storeSelected)
 	//Shouldn't be using a temporary camera.
 	//temporary cameras are only active during movement operations
 	ASSERT(!tempCam);
-	ASSERT(activeCam < cameras.size());
 	
 	
 	// Need to load a base name so that the other calls can replace it
@@ -749,7 +744,7 @@ unsigned int Scene::glSelect(bool storeSelected)
 	glPushMatrix();
 	//Apply the camera, but do NOT load the identity matrix, as
 	//we have set the pick matrix
-	cameras[activeCam]->apply(outWinAspect,boundCube,false);
+	activeCam->apply(outWinAspect,boundCube,false);
 
 	//Set up the objects. Only NON DISPLAYLIST items can be selected.
 	for(unsigned int ui=0; ui<objects.size(); ui++)
@@ -824,15 +819,15 @@ unsigned int Scene::glSelect(bool storeSelected)
 
 void Scene::finaliseCam()
 {
-	switch(cameras[activeCam]->type())
+	switch(activeCam->type())
 	{
 		case CAM_LOOKAT:
-			((CameraLookAt *)cameras[activeCam])->recomputeUpDirection();
+			((CameraLookAt *)activeCam)->recomputeUpDirection();
 			break;
 	}
 }
 
-void Scene::addSelectionDevices(const vector<SelectionDevice<Filter> *> &d)
+void Scene::addSelectionDevices(const vector<SelectionDevice *> &d)
 {
 	for(unsigned int ui=0;ui<d.size();ui++)
 	{
@@ -921,7 +916,7 @@ void Scene::applyDevice(float startX, float startY, float curX, float curY,
 	for(unsigned int ui=0;ui<activeBindings.size();ui++)
 	{
 		//Convert the mouse-XY coords into a world coordinate, depending upon mouse/
-		//key combinations
+		//key cobinations
 		Point3D worldVec;
 		Point3D vectorCoeffs[2];
 		activeBindings[ui]->computeWorldVectorCoeffs(mouseFlags,keyFlags,
@@ -952,16 +947,6 @@ void Scene::applyDevice(float startX, float startY, float curX, float curY,
 
 }
 
-unsigned int Scene::duplicateCameras(vector<Camera *> &cams) const
-{
-	cams.resize(cameras.size());
-
-	for(unsigned int ui=0;ui<cameras.size();ui++)	
-		cams[ui]=cameras[ui]->clone();
-
-	return activeCam;
-}	
-
 void Scene::getEffects(vector<const Effect *> &eff) const
 {
 	eff.resize(effects.size());
@@ -970,11 +955,6 @@ void Scene::getEffects(vector<const Effect *> &eff) const
 		eff[ui]=effects[ui];
 }	
 
-unsigned int Scene::getActiveCamId() const
-{
-	return camIDs.getId(activeCam);
-}
-
 
 void Scene::getModifiedBindings(std::vector<std::pair<const Filter *, SelectionBinding> > &bindings) const
 {
@@ -982,34 +962,22 @@ void Scene::getModifiedBindings(std::vector<std::pair<const Filter *, SelectionB
 		selectionDevices[ui]->getModifiedBindings(bindings);
 }
 
-void Scene::restrictView(float xS, float yS, float xFin, float yFin)
+void Scene::resetModifiedBindings()  
 {
-	viewRestrictStart[0]=xS;	
-	viewRestrictStart[1]=yS;	
-	
-	viewRestrictEnd[0]=xFin;	
-	viewRestrictEnd[1]=yFin;
-
-	viewRestrict=true;	
+	for(unsigned int ui=0;ui<selectionDevices.size();ui++)
+		selectionDevices[ui]->resetModifiedBindings();
 }
 
-bool Scene::isDefaultCam() const
+
+void Scene::setEffectVec(vector<Effect *> &e)
 {
-	return  activeCam==0;
+	clearEffects();
+
+	for(size_t ui=0;ui<e.size();ui++)
+		addEffect(e[ui]);
+	e.clear();
 }
 
-bool Scene::camNameExists(const std::string &s) const
-{
-	for(unsigned int ui=0;ui<cameras.size() ;ui++)
-	{
-		if(cameras[ui]->getUserString() == s)
-			return true;
-	}
-
-	return false;
-}
-
-
 
 unsigned int Scene::addEffect(Effect *e)
 {
@@ -1021,7 +989,6 @@ unsigned int Scene::addEffect(Effect *e)
 	return effectIDs.genId(effects.size()-1);
 }
 
-
 void Scene::removeEffect(unsigned int uniqueID)
 {
 	unsigned int position = effectIDs.getPos(uniqueID);
