@@ -20,43 +20,25 @@
 
 
 class Scene;
+class VisController;
+class Filter;
+class SelectionDevice;
+class SelectionBinding;
+
+#include "drawables.h"
 
 //Custom includes
-#include "../backend/viscontrol.h"
-
-#include "../backend/filter.h"
 #include "effect.h"
 
-//OpenGL debugging macro
-#if DEBUG
-#define glError() { \
-		GLenum err = glGetError(); \
-		while (err != GL_NO_ERROR) { \
-					fprintf(stderr, "glError: %s caught at %s:%u\n", (char *)gluErrorString(err), __FILE__, __LINE__); \
-					err = glGetError(); \
-				} \
-		std::cerr << "glErr Clean " << __FILE__ << ":" << __LINE__ << std::endl; \
-}
-#else
-#define glError()
-#endif
+#include "glDebug.h"
 
-#ifdef DEBUG
-	#define glStackDepths() { \
-		int gldepthdebug[3];glGetIntegerv (GL_MODELVIEW_STACK_DEPTH, gldepthdebug);\
-	       	glGetIntegerv (GL_PROJECTION_STACK_DEPTH, gldepthdebug+1);\
-	       	glGetIntegerv (GL_TEXTURE_STACK_DEPTH, gldepthdebug+2);\
-		std::cerr << "OpenGL Stack Depths: ModelV:" << gldepthdebug[0] << " Pr: "\
-		 << gldepthdebug[1] << " Tex:" << gldepthdebug[2] << std::endl;}
-#else
-	#define glStackDepths()
-#endif
 
 //!The scene class brings together elements such as objects, lights, and cameras
 //to enable scene rendering
 class Scene
 {
 	private:
+		bool glewInited;
 		//!Viscontroller. Needed for notification of updates during selection binding
 		VisController *visControl;
 		//!Objects that will be used for drawing
@@ -67,13 +49,12 @@ class Scene
 
 
 		//!Bindings for interactive object properties
-		std::vector<SelectionDevice<Filter> *> selectionDevices;
+		std::vector<SelectionDevice *> selectionDevices;
 
 		//!Various OpenGL effects
 		std::vector<const Effect *> effects;
 
-		//!Vector of camera stats
-		std::vector<Camera *> cameras;
+		Camera *activeCam;
 
 		//!Temporary override camera
 		Camera *tempCam;
@@ -84,20 +65,11 @@ class Scene
 		//!Size of window in px (needed if doing 2D drawing)
 		unsigned int winX,winY;
 
-		//!Which camera are we using
-		unsigned int activeCam;
 		//!Is there a camera set?
 		bool cameraSet;
 		//!Aspect ratio of output window (x/y) -- needed for cams
 		float outWinAspect;
 
-		//!Blank canvas colour
-		float r,g,b;
-
-
-		//!Camera id storage and handling
-		UniqueIDHandler camIDs;
-
 		//!Effect ID handler
 		UniqueIDHandler effectIDs;
 
@@ -113,14 +85,13 @@ class Scene
 		//!Tells us if we are in hover mode (should we draw hover overlays?)
 		bool hoverMode;
 
-		//!Is the camera to be restricted to only draw a particular portion of the viewport?
-		bool viewRestrict;
-
-		float viewRestrictStart[2], viewRestrictEnd[2];
-			
-		//!Last selected object from call to glSelect(). -1 if last call failed to identify an item
+		//!Last selected object from call to glSelect(). -1 if last
+		// call failed to identify an item
 		unsigned int lastSelected;
-	
+
+		//Prevent camera updates from being passed to opengl
+		bool witholdCamUpdate;
+
 		//!Last hoeverd object	
 		unsigned int lastHovered;
 
@@ -137,6 +108,17 @@ class Scene
 		//!Background colour
 		float rBack,gBack,bBack;
 
+		//!Should we a progress animation to the user in 3D?
+		bool showProgressAnimation;
+
+		//!Have we attempted to load the progress animation
+		bool attemptedLoadProgressAnim;
+
+		//texture to use for pgoress animation
+		DrawAnimatedOverlay progressAnimTex;
+
+		//Lighting vector
+		float lightPosition[4];
 
 		///!Draw the hover overlays
 		void drawHoverOverlay();
@@ -144,15 +126,21 @@ class Scene
 		//!Draw the normal overlays
 		void drawOverlays() const;
 
+		void drawProgressAnim() const;
+
 		//!initialise the drawing window
 		unsigned int initDraw();
 
-		void updateCam(const Camera *camToUse) const;
+		void updateCam(const Camera *camToUse, bool loadIdentity) const;
 
+		//reset the position of the overlay
+		void updateProgressOverlay(); 
 
 		//!Draw a specified vector of objects
 		void drawObjectVector(const std::vector<const DrawableObj*> &objects, bool &lightsOn, bool drawOpaques=true) const;
 
+		//!Disable copy constructor by making private
+		Scene &operator=(const Scene &);
 				
 	public:
 		//!Constructor
@@ -163,7 +151,7 @@ class Scene
 		//!Set the vis control
 		void setViscontrol(VisController *v) { visControl=v;};
 		//!Draw the objects in the active window. May adjust cameras and compute bounding as needed.
-		void draw();
+		void draw(bool noUpdateCam=false);
 
 	
 		//!clear rendering vectors
@@ -174,11 +162,19 @@ class Scene
 		void clearRefObjs();
 		//!Clear object bindings vector
 		void clearBindings();
-		//!Clear camera vector
-		void clearCams();
+	
+
+		//!Obtain the scene's light coordinates in camera relative space
+		// requires an array os size 4  (xyzw)
+		void getLightPos(float *f) const;
+		
+		//!Obtain the scene's light coordinates in camera relative space
+		// requires an array os size 4  (xyzw)
+		void setLightPos(const float *f);
+
 		//!Set the aspect ratio of the output window. Required.
 		void setAspect(float newAspect);
-		//!retreive aspect ratio (h/w) of output win
+		//!retrieve aspect ratio (h/w) of output win
 		float getAspect() const { return outWinAspect;};
 		
 		//!Add a drawable object 
@@ -188,30 +184,26 @@ class Scene
 		 */
 		void addDrawable(const DrawableObj *);
 		
-		//!Add a drawble to the refernce only section
-		/* Objects refferred to will not be modified or destroyed
-		 * by this class. It will onyl be used for drawing purposes
+		//!Add a drawable to the reference only section
+		/* Objects referred to will not be modified or destroyed
+		 * by this class. It will only be used for drawing purposes
 		 * It is up to the user to ensure that they are in a good state
 		 */
 		void addRefDrawable(const DrawableObj *);
-		
+	
+
+		bool setProgressAnimation(const vector<string> &animFiles);
+
+		void setShowProgress(bool show) ;
+
+		bool getShowProgress() const { return showProgressAnimation; }
 
 		//!remove a drawable object
 		void removeDrawable(unsigned int);
 
-		//!Add a camera 
-		/*!Pointer must be set to a valid (allocated) object.
-		 *!Scene will delete upon call to clearAll, clearCameras or
-		 *!upon destruction
-		 */
-		unsigned int addCam(Camera *);
-		
-		//!remove a camera object
-		void removeCam(unsigned int uniqueCamID);
-
-
+		void setActiveCam();
 		//! set the active camera
-		void setActiveCam(unsigned int uniqueCamID);
+		void setActiveCamByClone(const Camera *c);
 
 		//! get the active camera
 		Camera *getActiveCam() ;
@@ -240,25 +232,12 @@ class Scene
 		bool haveTempCam() const { return tempCam!=0;};
 
 		//!Clone the active camera
-		Camera *cloneActiveCam() const { return cameras[activeCam]->clone(); };
+		Camera *cloneActiveCam() const { return activeCam->clone(); };
 
-		//!Get the number of cameras (excluding tmp cam)
-		unsigned int getNumCams() const { return cameras.size(); } ;
-
-		//!Get the camera properties for a given camera
-		void getCamProperties(unsigned int uniqueID, CameraProperties &p) const;
-		//!Set the camera properties for a given camera. returns true if property set is OK
-		bool setCamProperty(unsigned int uniqueID, unsigned int key,
-	       					const std::string &value);
-		//!Return ALL the camera unique IDs
-		void getCameraIDs(vector<std::pair<unsigned int, std::string> > &idVec) const;
 
 		//!Modify the active camera position to ensure that scene is visible 
 		void ensureVisible(unsigned int direction);
 
-		//!Set the active camera to the first entry. Only to be called if getNumCams > 0, or if a camera is passed in
-		void setDefaultCam(Camera *c=0,bool setAsActive=true);
-
 		//!Call if user has stopped interacting with camera briefly.
 		void finaliseCam();
 
@@ -268,7 +247,7 @@ class Scene
 		unsigned int glSelect(bool storeSelection=true);
 
 		//!Add selection devices to the scene.
-		void addSelectionDevices(const std::vector<SelectionDevice<Filter> *> &d);
+		void addSelectionDevices(const std::vector<SelectionDevice *> &d);
 
 		//!Clear the current selection devices 
 		void clearDevices();
@@ -303,19 +282,11 @@ class Scene
 		//!Get a copy of the effects pointers
 		void getEffects(std::vector<const Effect *> &effects) const; 
 
-		//!Return the unique ID of the active camera
-		unsigned int getActiveCamId() const;
-
 		//!Return any devices that have been modified since their creation
 		void getModifiedBindings(std::vector<std::pair<const Filter *,SelectionBinding > > &bindings) const;
 
-		//!Restrict the openGL drawing view when using the camera
-		void restrictView(float xS,float yS, float xFin, float yFin);
-		//!Disable view restriction
-		void unrestrictView() { viewRestrict=false;};
-
-		//!True if the current camera is the default (0th) camera
-		bool isDefaultCam() const; 
+		//!Reset any modifiecations to bindings back to the unmodified state
+		void resetModifiedBindings();
 
 		//!Set whether to use alpha blending
 		void setAlpha(bool newAlpha) { useAlpha=newAlpha;};
@@ -329,14 +300,11 @@ class Scene
 		bool getWorldAxisVisible() const { return showAxis;};
 
 		//!Set window size
-		void setWinSize(unsigned int x, unsigned int y) {winX=x;winY=y;}
+		void setWinSize(unsigned int x, unsigned int y) {winX=x;winY=y; updateProgressOverlay();}
 
 		//!Get the scene boundinng box
 		BoundCube getBound() const { return boundCube;}
 
-		//!Returns true if this camera name is already in use
-		bool camNameExists(const std::string &s) const;
-
 		//!Set the background colour
 		void setBackgroundColour(float newR,float newG,float newB) { rBack=newR;gBack=newG;bBack=newB;};
 
@@ -351,6 +319,13 @@ class Scene
 		//!Set whether to use effects or not
 		void setEffects(bool enable) {useEffects=enable;} 
 
+		//!Set the effect vector
+		/*! Pointers will become "owned" by scene
+		 * and will be deleted during destruction, clear, or next setEffectVec call
+		 * input vector will be cleared.
+		 */
+		void setEffectVec(vector<Effect *> &e);
+
 		//!Add an effect
 		unsigned int addEffect(Effect *e);
 		//!Remove a given effect
