@@ -18,9 +18,9 @@
 
 
 #include "ExportPos.h"
-#include "wxcommon.h"
+#include "wx/wxcommon.h"
 
-#include "wxcomponents.h"
+#include "wx/wxcomponents.h"
 #include "common/translation.h"
 
 #include <wx/treectrl.h>
@@ -151,7 +151,7 @@ void ExportPosDialog::initialiseData(FilterTree &f)
 
 	ProgressData p;
 	//TODO: Is trashing the devices a problem? do we have to restore them??
-	std::vector<SelectionDevice<Filter> *> dummyDevices;
+	std::vector<SelectionDevice *> dummyDevices;
 	std::vector<std::pair<const Filter *, string > > consoleStrings;
 	filterTree.refreshFilterTree(outputData,dummyDevices,consoleStrings,p,yieldCallback);
 
