@@ -18,10 +18,11 @@
 
 #include "StashDialog.h"
 
-#include "wxcommon.h"
-#include "wxcomponents.h"
+#include "wx/wxcommon.h"
+#include "wx/wxcomponents.h"
 #include "common/translation.h"
 
+#include "./backend/viscontrol.h"
 
 using std::pair;
 using std::string;
@@ -47,7 +48,7 @@ StashDialog::StashDialog(wxWindow* parent, int id, const wxString& title, const
     label_6 = new wxStaticText(this, wxID_ANY, wxTRANS("Stashed Tree"));
     treeFilters = new wxTreeCtrl(this, ID_TREE_FILTERS, wxDefaultPosition, wxDefaultSize, wxTR_HAS_BUTTONS|wxTR_NO_LINES|wxTR_DEFAULT_STYLE|wxSUNKEN_BORDER|wxTR_HIDE_ROOT);
     label_7 = new wxStaticText(this, wxID_ANY, wxTRANS("Properties"));
-    gridProperties = new wxPropertyGrid(this, ID_GRID_FILTER);
+    gridProperties = new wxCustomPropGrid(this, ID_GRID_FILTER);
     btnOK = new wxButton(this, wxID_OK, wxEmptyString);
 
     set_properties();
@@ -113,7 +114,7 @@ void StashDialog::OnListKeyDown(wxListEvent &event)
 				if ( item == -1 )
 					break;
 
-				visControl->deleteStash(listStashes->GetItemData(item));
+				visControl->eraseStash(listStashes->GetItemData(item));
 			}
 			
 			//Update the filter list
@@ -353,7 +354,7 @@ void StashDialog::OnBtnRemove(wxCommandEvent &event)
 		if ( item == -1 )
 			break;
 
-		visControl->deleteStash(listStashes->GetItemData(item));
+		visControl->eraseStash(listStashes->GetItemData(item));
 		updateList();
 		updateTree();
 		updateGrid();
