@@ -18,11 +18,11 @@
 
 #include <wx/wx.h>
 #include <wx/dcbuffer.h>
-#include "wxcomponents.h"
+#include "wx/wxcomponents.h"
 
 #include "mathglPane.h"
 
-#include "wxcommon.h"
+#include "wx/wxcommon.h"
 #include "common/translation.h"
 
 #ifdef USE_MGL2
@@ -64,6 +64,7 @@ BEGIN_EVENT_TABLE(MathGLPane, wxPanel)
 	EVT_RIGHT_DOWN(MathGLPane::rightClick)
 	EVT_LEAVE_WINDOW(MathGLPane::mouseLeftWindow)
 	EVT_LEFT_DCLICK(MathGLPane::mouseDoubleLeftClick) 
+	EVT_MIDDLE_DCLICK(MathGLPane::mouseDoubleLeftClick) 
 	EVT_SIZE(MathGLPane::resized)
 	EVT_KEY_DOWN(MathGLPane::keyPressed)
 	EVT_KEY_UP(MathGLPane::keyReleased)
@@ -86,11 +87,14 @@ wxPanel(parent, id,  wxDefaultPosition, wxDefaultSize)
 	COMPILE_ASSERT(THREEDEP_ARRAYSIZE(MOUSE_ACTION_NEEDS_REDRAW) == MOUSE_MODE_ENUM_END);
 
 	hasResized=true;
-	limitInteract=haveUpdates=false;
+	limitInteract=false;
 	mouseDragMode=MOUSE_MODE_ENUM_END;	
 	leftWindow=true;
 	thePlot=0;	
 	gr=0;
+	ownPlotPtr=false;
+	lastEditedPlot=lastEditedRegion=-1;
+	regionSelfUpdate=false;
 
 	SetBackgroundStyle(wxBG_STYLE_CUSTOM);
 
@@ -98,13 +102,19 @@ wxPanel(parent, id,  wxDefaultPosition, wxDefaultSize)
 
 MathGLPane::~MathGLPane()
 {
-	if(thePlot)
+	if(thePlot && ownPlotPtr)
 		delete thePlot;
 	if(gr)
 		delete gr;
 }
 
 
+bool MathGLPane::readyForInput() const
+{
+	return (thePlot && gr && 
+		!thePlot->isInteractionLocked()  && thePlot->getNumTotal());
+}
+
 unsigned int MathGLPane::getAxisMask(int x, int y) const
 {
 	
@@ -141,12 +151,13 @@ unsigned int MathGLPane::getAxisMask(int x, int y) const
 	return retVal;
 }
 
-void MathGLPane::setPlotWrapper(PlotWrapper *newPlot)
+void MathGLPane::setPlotWrapper(PlotWrapper *newPlot,bool takeOwnPtr)
 {
-	if(thePlot)
+	if(thePlot && ownPlotPtr)
 		delete thePlot;
 
 	thePlot=newPlot;
+	ownPlotPtr=takeOwnPtr;
 
 	Refresh();
 }
@@ -158,7 +169,7 @@ void MathGLPane::render(wxPaintEvent &event)
 	
     wxAutoBufferedPaintDC   *dc=new wxAutoBufferedPaintDC(this);
 
-	if(!thePlot || !plotSelList || thePlot->isInteractionLocked() )
+	if(!thePlot || thePlot->isInteractionLocked() )
 	{
 		delete dc;
 		return;
@@ -179,10 +190,8 @@ void MathGLPane::render(wxPaintEvent &event)
 
 
 	//Set the enabled and disabled plots
-	wxArrayInt a;
-
-	unsigned int nItems =plotSelList->GetSelections(a);
-
+	unsigned int nItems=thePlot->getNumVisible();
+	
 	if(!nItems)
 	{
 #ifdef __WXGTK__
@@ -213,23 +222,8 @@ void MathGLPane::render(wxPaintEvent &event)
 
 	}
 
-	//Set which plots should be visible
-	thePlot->hideAll();
-	std::vector<unsigned int> newVisible;
-	for(unsigned int ui=0;ui<nItems; ui++)
-	{
-		unsigned int plotID;
-		wxListUint *l =(wxListUint *) plotSelList->GetClientObject(a[ui]);
-		plotID = l->value;
-		newVisible.push_back(ui);
-		thePlot->setVisible(plotID,true);
-	}
-
-
-
-
 	//If the plot has changed, been resized or is performing
-	// a mouse action that requires updating, we need to update it
+	// a mouse action that reuqires updating, we need to update it
 	//likewise if we don't have a plot, we need one.
 	if(!gr || hasChanged || hasResized || 
 		MOUSE_ACTION_NEEDS_REDRAW[mouseDragMode])
@@ -274,15 +268,6 @@ void MathGLPane::render(wxPaintEvent &event)
 		hasResized=false;
 	}
 
-	//If the visibility hasn't changed, then reset the 
-	//plot to "no changes" state. Otherwise,
-	//swap the visibility vectors
-	if(lastVisible.size() == newVisible.size() &&
-		std::equal(lastVisible.begin(),lastVisible.end(),newVisible.begin()))
-		thePlot->resetChange();
-	else
-		lastVisible.swap(newVisible);
-
 	//Copy the plot's memory buffer into a wxImage object, then draw it	
 #ifdef USE_MGL2
 	char *rgbdata = (char*)malloc(w*h*3);
@@ -329,8 +314,8 @@ void MathGLPane::render(wxPaintEvent &event)
 
 			const int END_MARKER_SIZE=5;
 
-		//If the cursor is wholly below
-			//the axis, draw a line rather than a box
+			//If the cursor is wholly below
+			//the axis, draw a line rather than abox
 
 			unsigned int startMask, endMask;
 			startMask=getAxisMask(draggingStart.x, draggingStart.y);
@@ -412,8 +397,8 @@ void MathGLPane::updateMouseCursor()
 		return;
 
 	//Set cursor to normal by default
-		SetCursor(wxNullCursor);
-	if(!thePlot->getNumVisible())
+	SetCursor(wxNullCursor);
+	if(!readyForInput())
 		return;
 
 	//Update mouse cursor
@@ -462,7 +447,8 @@ bool MathGLPane::getRegionUnderCursor(const wxPoint  &mousePos, unsigned int &pl
 
 	//Convert the mouse coordinates to data coordinates.
 	mglPoint pMouse= gr->CalcXYZ(mousePos.x,mousePos.y);
-
+	if(!readyForInput())
+		return false;
 
 	//Only allow  range interaction within the plot bb
 #ifdef USE_MGL2
@@ -483,7 +469,7 @@ bool MathGLPane::getRegionUnderCursor(const wxPoint  &mousePos, unsigned int &pl
 void MathGLPane::mouseMoved(wxMouseEvent& event)
 {
 	leftWindow=false;
-	if(!thePlot || !gr || thePlot->isInteractionLocked() )
+	if(!readyForInput())
 	{
 		mouseDragMode=MOUSE_MODE_ENUM_END;
 		return;
@@ -526,7 +512,7 @@ void MathGLPane::mouseMoved(wxMouseEvent& event)
 
 void MathGLPane::mouseDoubleLeftClick(wxMouseEvent& event)
 {
-	if(!thePlot || !gr || thePlot->isInteractionLocked())
+	if(!readyForInput())
 		return;
 
 	//Cancel any mouse drag mode
@@ -563,6 +549,11 @@ void MathGLPane::mouseDoubleLeftClick(wxMouseEvent& event)
 	Refresh();
 }
 
+void MathGLPane::mouseDoubleMiddleClick(wxMouseEvent &event)
+{
+	mouseDoubleLeftClick(event);
+}
+
 
 void MathGLPane::oneDMouseDownAction(bool leftDown,bool middleDown,
 		 bool alternateDown, int dragX,int dragY)
@@ -623,7 +614,7 @@ void MathGLPane::oneDMouseDownAction(bool leftDown,bool middleDown,
 
 void MathGLPane::leftMouseDown(wxMouseEvent& event)
 {
-	if(!gr || !thePlot->getNumVisible() || thePlot->isInteractionLocked())
+	if(!readyForInput())
 		return;
 
 	int w,h;
@@ -652,7 +643,7 @@ void MathGLPane::leftMouseDown(wxMouseEvent& event)
 
 void MathGLPane::middleMouseDown(wxMouseEvent &event)
 {
-	if(!gr || !thePlot->getNumVisible() || thePlot->isInteractionLocked())
+	if(!readyForInput())
 		return;
 	
 	int w,h;
@@ -683,7 +674,7 @@ void MathGLPane::middleMouseDown(wxMouseEvent &event)
 void MathGLPane::mouseWheelMoved(wxMouseEvent& event)
 {
 	//If no valid plot, don't do anything
-	if(!thePlot || !gr || thePlot->isInteractionLocked() )
+	if(!readyForInput())
 		return;
 	
 	//no action if currently dragging
@@ -788,9 +779,12 @@ void MathGLPane::mouseWheelMoved(wxMouseEvent& event)
 void MathGLPane::leftMouseReleased(wxMouseEvent& event)
 {
 
-	if(!thePlot || !gr || thePlot->isInteractionLocked() )
+	if(!readyForInput())
 		return;
 
+	//!Do we have region updates?
+	bool haveUpdates=false;
+	
 	switch(mouseDragMode)
 	{
 		case MOUSE_MODE_DRAG:
@@ -807,10 +801,13 @@ void MathGLPane::leftMouseReleased(wxMouseEvent& event)
 				//we need to tell viscontrol that we have done a region
 				//update
 				mglPoint mglCurMouse= gr->CalcXYZ(curMouse.x,curMouse.y);
+				lastEditedRegion=startMouseRegion;
+				lastEditedPlot=startMousePlot;
 			
 				//Send the movement to the parent filter
-				thePlot->moveRegion(startMousePlot,
-					startMouseRegion,regionMoveType,mglCurMouse.x,mglCurMouse.y);	
+				thePlot->moveRegion(startMousePlot,startMouseRegion,
+							regionSelfUpdate,regionMoveType,
+								mglCurMouse.x,mglCurMouse.y);	
 				haveUpdates=true;	
 
 			}
@@ -821,14 +818,31 @@ void MathGLPane::leftMouseReleased(wxMouseEvent& event)
 		;
 	}
 
+
+
+
 	mouseDragMode=MOUSE_MODE_ENUM_END;
 	Refresh();
+	
+	if(haveUpdates)
+	{
+		for(size_t ui=0;ui<updateHandlers.size(); ui++)
+		{
+			pair<wxWindow*,UpdateHandler> u;
+			u=updateHandlers[ui];
+
+			//Call the function
+			UpdateHandler h = u.second;
+			wxWindow *w=u.first;
+			(w->*h)();
+		}
+	}
 }
 
 void MathGLPane::middleMouseReleased(wxMouseEvent& event)
 {
 
-	if(!gr || thePlot->isInteractionLocked())
+	if(!readyForInput())
 		return;
 
 	if(mouseDragMode == MOUSE_MODE_DRAG_PAN)
@@ -970,14 +984,14 @@ void MathGLPane::mouseLeftWindow(wxMouseEvent& event)
 
 void MathGLPane::keyPressed(wxKeyEvent& event) 
 {
-	if(!gr || thePlot->isInteractionLocked() )
+	if(!readyForInput())
 		return;
 	updateMouseCursor();
 }
 
 void MathGLPane::keyReleased(wxKeyEvent& event) 
 {
-	if(!gr || thePlot->isInteractionLocked() )
+	if(!readyForInput())
 		return;
 	updateMouseCursor();
 }
@@ -1242,8 +1256,9 @@ void MathGLPane::drawRegionDraggingOverlay(wxDC *dc) const
 	float regionLimitX,regionLimitY;
 	regionLimitX=mglCurMouse.x;
 	regionLimitY=mglCurMouse.y;
+
 	//See where extending the region is allowed up to.
-	thePlot->moveRegionLimit(startMousePlot,startMouseRegion,
+	thePlot->findRegionLimit(startMousePlot,startMouseRegion,
 					regionMoveType, regionLimitX,regionLimitY);
 	
 	int deltaDrag;
